﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace S3UploadDownload
{
    public partial class FormUpload : Form
    {
        SprightlySoftAWS.S3.CalculateHash MyCalculateHash;
        SprightlySoftAWS.S3.Upload MyUpload;
        System.ComponentModel.BackgroundWorker UploadBackgroundWorker;
        System.ComponentModel.BackgroundWorker CalculateHashBackgroundWorker;

        public FormUpload()
        {
            InitializeComponent();
        }

        private void FormUpload_Load(object sender, EventArgs e)
        {
            Properties.Settings MySettings = new Properties.Settings();

            TextBoxUploadFileName.Text = MySettings.UploadFileName;
            TextBoxUploadBucketName.Text = MySettings.UploadBucketName;
            TextBoxUploadKeyName.Text = MySettings.UploadKeyName;
            TextBoxUploadContentType.Text = MySettings.UploadContentType;
            CheckBoxUploadMakePublic.Checked = MySettings.UploadMakePublic;

            MyCalculateHash = new SprightlySoftAWS.S3.CalculateHash();
            MyCalculateHash.ProgressChangedEvent += MyCalculateHash_ProgressChangedEvent;

            MyUpload = new SprightlySoftAWS.S3.Upload();
            MyUpload.ProgressChangedEvent += MyUpload_ProgressChangedEvent;

            CalculateHashBackgroundWorker = new System.ComponentModel.BackgroundWorker();
            CalculateHashBackgroundWorker.DoWork += CalculateHashBackgroundWorker_DoWork;
            CalculateHashBackgroundWorker.RunWorkerCompleted += CalculateHashBackgroundWorker_RunWorkerCompleted;

            UploadBackgroundWorker = new System.ComponentModel.BackgroundWorker();
            UploadBackgroundWorker.DoWork += UploadBackgroundWorker_DoWork;
            UploadBackgroundWorker.RunWorkerCompleted += UploadBackgroundWorker_RunWorkerCompleted;
            
            LabelAction.Text = "Waiting";
            ProgressBarTransfered.Value = 0;
            LabelBytesTransfered.Text = "0 bytes / 0 bytes";
        }

        protected override void OnFormClosing(FormClosingEventArgs e)
        {
            Properties.Settings MySettings = new Properties.Settings();

            MySettings.UploadFileName = TextBoxUploadFileName.Text;
            MySettings.UploadBucketName = TextBoxUploadBucketName.Text;
            MySettings.UploadKeyName = TextBoxUploadKeyName.Text;
            MySettings.UploadContentType = TextBoxUploadContentType.Text;
            MySettings.UploadMakePublic = CheckBoxUploadMakePublic.Checked;

            MySettings.Save();
        }

        private void EnableDisableStart()
        {
            TextBoxUploadFileName.Enabled = false;
            TextBoxUploadBucketName.Enabled = false;
            TextBoxUploadKeyName.Enabled = false;
            TextBoxUploadContentType.Enabled = false;
            CheckBoxUploadMakePublic.Enabled = false;
            ButtonUploadBrowse.Enabled = false;
            ButtonUploadFile.Enabled = false;

            ButtonAbort.Enabled = true;
            LabelAction.Enabled = true;
            ProgressBarTransfered.Enabled = true;
            LabelBytesTransfered.Enabled = true;

            LabelAction.Text = "Waiting";
            ProgressBarTransfered.Value = 0;
            LabelBytesTransfered.Text = "0 bytes / 0 bytes";
        }

        private void EnableDisableEnd()
        {
            TextBoxUploadFileName.Enabled = true;
            TextBoxUploadBucketName.Enabled = true;
            TextBoxUploadKeyName.Enabled = true;
            TextBoxUploadContentType.Enabled = true;
            CheckBoxUploadMakePublic.Enabled = true;
            ButtonUploadBrowse.Enabled = true;
            ButtonUploadFile.Enabled = true;

            ButtonAbort.Enabled = false;
            LabelAction.Enabled = false;
            ProgressBarTransfered.Enabled = false;
            LabelBytesTransfered.Enabled = false;

            LabelAction.Text = "Waiting";
            ProgressBarTransfered.Value = 0;
            LabelBytesTransfered.Text = "0 bytes / 0 bytes";
        }

        private void  MyCalculateHash_ProgressChangedEvent()
        {
            if (this.InvokeRequired == true)
            {
                this.BeginInvoke(new MethodInvoker(delegate() { MyCalculateHash_ProgressChangedEvent(); }));
            }
            else
            {
                //Update the progress bar when the ProgressChangedEvent is fired.
                if (MyCalculateHash.BytesTotal > 0)
                {
                    decimal MyDecimal = (Convert.ToDecimal(MyCalculateHash.BytesProcessed) / Convert.ToDecimal(MyCalculateHash.BytesTotal)) * 100;
                    ProgressBarTransfered.Value = Convert.ToInt32(MyDecimal);

                    SprightlySoftAWS.S3.Helper MyHelper = new SprightlySoftAWS.S3.Helper();
                    LabelBytesTransfered.Text = MyHelper.FormatByteSize(MyCalculateHash.BytesProcessed) + " / " + MyHelper.FormatByteSize(MyCalculateHash.BytesTotal);
                }
            }
        }

        private void  MyUpload_ProgressChangedEvent()
        {
            if (this.InvokeRequired == true)
            {
                this.BeginInvoke(new MethodInvoker(delegate() { MyUpload_ProgressChangedEvent(); }));
            }
            else
            {
                //Update the progress bar when the ProgressChangedEvent is fired.
                if (MyUpload.BytesTotal > 0)
                {
                    decimal MyDecimal = (Convert.ToDecimal(MyUpload.BytesTransfered) / Convert.ToDecimal(MyUpload.BytesTotal)) * 100;
                    ProgressBarTransfered.Value = Convert.ToInt32(MyDecimal);

                    SprightlySoftAWS.S3.Helper MyHelper = new SprightlySoftAWS.S3.Helper();
                    LabelBytesTransfered.Text = MyHelper.FormatByteSize(MyUpload.BytesTransfered) + " / " + MyHelper.FormatByteSize(MyUpload.BytesTotal);
                }
            }
        }

        private void CalculateHashBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            //Call the CalculateMD5FromFile function and set the result.  When the function is complete
            //the RunWorkerCompleted event will fire.  Use the LocalFileName value from the passed hash table.
            System.Collections.Hashtable CalculateHashHashTable = e.Argument as System.Collections.Hashtable;
            e.Result = MyCalculateHash.CalculateMD5FromFile(CalculateHashHashTable["LocalFileName"].ToString());
        }

        private void CalculateHashBackgroundWorker_RunWorkerCompleted(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
        {
            //If the CalculateMD5FromFile function was successful upload the file.
            if (MyCalculateHash.ErrorNumber == 0)
            {
                LabelAction.Text = "Uploading file";
                Application.DoEvents();

                //Set the extra request headers to send with the upload
                Dictionary<String, String> ExtraRequestHeaders = new Dictionary<String, String>();

                if (TextBoxUploadContentType.Text != "")
                {
                    ExtraRequestHeaders.Add("Content-Type", TextBoxUploadContentType.Text);
                }

                if (CheckBoxUploadMakePublic.Checked == true)
                {
                    ExtraRequestHeaders.Add("x-amz-acl", "public-read");
                }

                //Use the MD5 hash that was calculated previously.
                ExtraRequestHeaders.Add("Content-MD5", e.Result.ToString());

                Properties.Settings MySettings = new Properties.Settings();

                String RequestURL;
                RequestURL = MyUpload.BuildS3RequestURL(true, "s3.amazonaws.com", TextBoxUploadBucketName.Text, TextBoxUploadKeyName.Text, "");

                String RequestMethod = "PUT";

                ExtraRequestHeaders.Add("x-amz-date", DateTime.UtcNow.ToString("r"));

                String AuthorizationValue;
                AuthorizationValue = MyUpload.GetS3AuthorizationValue(RequestURL, RequestMethod, ExtraRequestHeaders, MySettings.AWSAccessKeyId, MySettings.AWSSecretAccessKey);
                ExtraRequestHeaders.Add("Authorization", AuthorizationValue);

                //Create a hash table of of parameters to sent to the upload function.
                System.Collections.Hashtable UploadHashTable = new System.Collections.Hashtable();
                UploadHashTable.Add("RequestURL", RequestURL);
                UploadHashTable.Add("RequestMethod", RequestMethod);
                UploadHashTable.Add("ExtraRequestHeaders", ExtraRequestHeaders);
                UploadHashTable.Add("LocalFileName", TextBoxUploadFileName.Text);

                //Run the UploadFile call in a BackgroundWorker to prevent the Window from freezing.
                UploadBackgroundWorker.RunWorkerAsync(UploadHashTable);
            }
            else
            {
                EnableDisableEnd();
                MessageBox.Show(MyCalculateHash.ErrorDescription, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonUploadFile_Click(object sender, EventArgs e)
        {
            EnableDisableStart();

            if (System.IO.File.Exists(TextBoxUploadFileName.Text) == true)
            {
                //Start by calculating the MD5 hash of the file.  This will be sent to Amazon in the 
                //upload to make sure it arrives correctly.
                LabelAction.Text = "Calculating MD5 hash";
                Application.DoEvents();

                //Run the hash calculation in a BackgroundWorker process.  Calculating the hash of a
                //large file will take a while.  Running the process in a BackgroundWorker will prevent
                //the form from locking up.

                //Use a hash table to pass parameters to the function in the BackgroundWorker.
                System.Collections.Hashtable CalculateHashHashTable = new System.Collections.Hashtable();
                CalculateHashHashTable.Add("LocalFileName", TextBoxUploadFileName.Text);

                CalculateHashBackgroundWorker.RunWorkerAsync(CalculateHashHashTable);
            }
            else
            {
                EnableDisableEnd();
                MessageBox.Show("The local file does not exist.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonUploadBrowse_Click(object sender, EventArgs e)
        {
            System.Windows.Forms.OpenFileDialog OpenFileDialog = new System.Windows.Forms.OpenFileDialog();
            OpenFileDialog.Multiselect = false;
            OpenFileDialog.FileName = "";
            OpenFileDialog.ShowDialog();

            if (OpenFileDialog.FileName != "" && System.IO.File.Exists(OpenFileDialog.FileName) == true)
            {
                TextBoxUploadFileName.Text = OpenFileDialog.FileName;
                TextBoxUploadKeyName.Text = System.IO.Path.GetFileName(OpenFileDialog.FileName);

                //set the content type from the file's extension
                //the SprightlySoftS3 Helper class contains a function that returns a dictionary of extensions and associated content types
                SprightlySoftAWS.S3.Helper MyHelper = new SprightlySoftAWS.S3.Helper();
                Dictionary<String, String> ContentTypesDictionary;
                ContentTypesDictionary = MyHelper.GetContentTypesDictionary();

                String MyExtension;
                MyExtension = System.IO.Path.GetExtension(OpenFileDialog.FileName);
                MyExtension = MyExtension.ToLower();

                if (ContentTypesDictionary.ContainsKey(MyExtension) == true)
                {
                    TextBoxUploadContentType.Text = ContentTypesDictionary[MyExtension];
                }
                else
                {
                    TextBoxUploadContentType.Text = "";
                }
            }
        }

        private void UploadBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            //Run the UploadFile call.
            System.Collections.Hashtable UploadHashTable = e.Argument as System.Collections.Hashtable;

            e.Result = MyUpload.UploadFile(UploadHashTable["RequestURL"].ToString(), UploadHashTable["RequestMethod"].ToString(), UploadHashTable["ExtraRequestHeaders"] as Dictionary<String, String>, UploadHashTable["LocalFileName"].ToString());
        }

        private void UploadBackgroundWorker_RunWorkerCompleted(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
        {
            System.Diagnostics.Debug.Print("");
            System.Diagnostics.Debug.Print(MyUpload.LogData);
            System.Diagnostics.Debug.Print("");

            EnableDisableEnd();

            if (Convert.ToBoolean(e.Result) == true)
            {
                MessageBox.Show("Upload complete.", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                //Show the error message.
                String ResponseMessage;

                if (MyUpload.ResponseString == "")
                {
                    ResponseMessage = MyUpload.ErrorDescription;
                }
                else
                {
                    System.Xml.XmlDocument XmlDoc = new System.Xml.XmlDocument();
                    XmlDoc.LoadXml(MyUpload.ResponseString);

                    System.Xml.XmlNode XmlNode;
                    XmlNode = XmlDoc.SelectSingleNode("/Error/Message");

                    ResponseMessage = XmlNode.InnerText;
                }

                MessageBox.Show(ResponseMessage, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonAbort_Click(object sender, EventArgs e)
        {
            //Abort the CalculateHash call or Upload call depending on what is taking place.
            if (MyCalculateHash.State == 1)
            {
                MyCalculateHash.Abort();
            }
            else if (MyUpload.State == 1)
            {
                MyUpload.Abort();
            }
        }

    }
}
