﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace S3UploadDownload
{
    public partial class FormUploadEstimate : Form
    {

        SprightlySoftAWS.S3.CalculateHash MyCalculateHash;
        SprightlySoftAWS.S3.UploadEstimate MyUpload;
        System.ComponentModel.BackgroundWorker UploadBackgroundWorker;
        System.ComponentModel.BackgroundWorker CalculateHashBackgroundWorker;
        System.ComponentModel.BackgroundWorker ETagBackgroundWorker;
        System.Timers.Timer TimerUpdateStatus;
        FormConflictUpload MyFormConflictUpload;
        
        public FormUploadEstimate()
        {
            InitializeComponent();
        }

        private void FormUploadEstimate_Load(object sender, EventArgs e)
        {
            Properties.Settings MySettings = new Properties.Settings();

            TextBoxUploadFileName.Text = MySettings.UploadEstimateFileName;
            TextBoxUploadBucketName.Text = MySettings.UploadEstimateBucketName;
            TextBoxUploadKeyName.Text = MySettings.UploadEstimateKeyName;
            TextBoxUploadContentType.Text = MySettings.UploadEstimateContentType;
            CheckBoxUploadMakePublic.Checked = MySettings.UploadEstimateMakePublic;

            MyCalculateHash = new SprightlySoftAWS.S3.CalculateHash();

            MyUpload = new SprightlySoftAWS.S3.UploadEstimate();
            MyUpload.ProgressChangedEvent += MyUpload_ProgressChangedEvent;

            UploadBackgroundWorker = new System.ComponentModel.BackgroundWorker();
            UploadBackgroundWorker.DoWork += UploadBackgroundWorker_DoWork;
            UploadBackgroundWorker.RunWorkerCompleted += UploadBackgroundWorker_RunWorkerCompleted;

            CalculateHashBackgroundWorker = new System.ComponentModel.BackgroundWorker();
            CalculateHashBackgroundWorker.DoWork += CalculateHashBackgroundWorker_DoWork;
            CalculateHashBackgroundWorker.RunWorkerCompleted += CalculateHashBackgroundWorker_RunWorkerCompleted;

            ETagBackgroundWorker = new System.ComponentModel.BackgroundWorker();
            ETagBackgroundWorker.DoWork += ETagBackgroundWorker_DoWork;
            ETagBackgroundWorker.RunWorkerCompleted += ETagBackgroundWorker_RunWorkerCompleted;

            //use a timer to show the current speed and time remaining
            TimerUpdateStatus = new System.Timers.Timer();
            TimerUpdateStatus.Elapsed += new System.Timers.ElapsedEventHandler(TimerUpdateStatus_Tick);
            TimerUpdateStatus.Interval = 500;
        }

        protected override void OnFormClosing(FormClosingEventArgs e)
        {
            Properties.Settings MySettings = new Properties.Settings();

            MySettings.UploadEstimateFileName = TextBoxUploadFileName.Text;
            MySettings.UploadEstimateBucketName = TextBoxUploadBucketName.Text;
            MySettings.UploadEstimateKeyName = TextBoxUploadKeyName.Text;
            MySettings.UploadEstimateContentType = TextBoxUploadContentType.Text;
            MySettings.UploadEstimateMakePublic = CheckBoxUploadMakePublic.Checked;

            MySettings.Save();
        }

        private void EnableDisableStart()
        {
            TextBoxUploadFileName.Enabled = false;
            TextBoxUploadBucketName.Enabled = false;
            TextBoxUploadKeyName.Enabled = false;
            TextBoxUploadContentType.Enabled = false;
            CheckBoxUploadMakePublic.Enabled = false;
            ButtonUploadBrowse.Enabled = false;
            ButtonUploadFile.Enabled = false;
            ButtonAbort.Enabled = true;

            ToolStripStatusLabelAction.Text = "";
            ToolStripProgressBar.Value = 0;
            ToolStripStatusLabelBytes.Text = "";
            ToolStripStatusLabelSpeed.Text = "";
            ToolStripStatusLabelElapsed.Text = "";
            ToolStripStatusLabelRemaining.Text = "";

            TimerUpdateStatus.Enabled = true;
        }

        private void EnableDisableEnd()
        {
            TextBoxUploadFileName.Enabled = true;
            TextBoxUploadBucketName.Enabled = true;
            TextBoxUploadKeyName.Enabled = true;
            TextBoxUploadContentType.Enabled = true;
            CheckBoxUploadMakePublic.Enabled = true;
            ButtonUploadBrowse.Enabled = true;
            ButtonUploadFile.Enabled = true;
            ButtonAbort.Enabled = false;

            ToolStripStatusLabelAction.Visible = false;
            ToolStripProgressBar.Visible = false;
            ToolStripStatusLabelBytes.Visible = false;
            ToolStripStatusLabelSpeed.Visible = false;
            ToolStripStatusLabelElapsed.Visible = false;
            ToolStripStatusLabelRemaining.Visible = false;

            TimerUpdateStatus.Enabled = false;
        }

        private void ButtonUploadBrowse_Click(object sender, EventArgs e)
        {
            System.Windows.Forms.OpenFileDialog OpenFileDialog = new System.Windows.Forms.OpenFileDialog();
            OpenFileDialog.Multiselect = false;
            OpenFileDialog.FileName = "";
            OpenFileDialog.ShowDialog();

            if (OpenFileDialog.FileName != "" && System.IO.File.Exists(OpenFileDialog.FileName) == true)
            {
                TextBoxUploadFileName.Text = OpenFileDialog.FileName;
                TextBoxUploadKeyName.Text = System.IO.Path.GetFileName(OpenFileDialog.FileName);

                //set the content type from the file's extension
                //the SprightlySoftS3 Helper class contains a function that returns a dictionary of extensions and associated content types
                SprightlySoftAWS.S3.Helper MyHelper = new SprightlySoftAWS.S3.Helper();
                Dictionary<String, String> ContentTypesDictionary;
                ContentTypesDictionary = MyHelper.GetContentTypesDictionary();

                String MyExtension;
                MyExtension = System.IO.Path.GetExtension(OpenFileDialog.FileName);
                MyExtension = MyExtension.ToLower();

                if (ContentTypesDictionary.ContainsKey(MyExtension) == true)
                {
                    TextBoxUploadContentType.Text = ContentTypesDictionary[MyExtension];
                }
                else
                {
                    TextBoxUploadContentType.Text = "";
                }
            }
        }

        private void ButtonUploadFile_Click(object sender, EventArgs e)
        {
            EnableDisableStart();

            if (System.IO.File.Exists(TextBoxUploadFileName.Text) == true)
            {
                ToolStripStatusLabelAction.Text = "Checking if file exists....";
                ToolStripStatusLabelAction.Visible = true;
                Application.DoEvents();

                Properties.Settings MySettings = new Properties.Settings();

                //check if the file exists on amazon
                SprightlySoftAWS.REST MyREST = new SprightlySoftAWS.REST();

                String RequestURL;
                RequestURL = MyREST.BuildS3RequestURL(true, "s3.amazonaws.com", TextBoxUploadBucketName.Text, TextBoxUploadKeyName.Text, "");

                String RequestMethod = "HEAD";

                Dictionary<String, String> ExtraRequestHeaders = new Dictionary<String, String>();
                ExtraRequestHeaders.Add("x-amz-date", DateTime.UtcNow.ToString("r"));

                String AuthorizationValue;
                AuthorizationValue = MyREST.GetS3AuthorizationValue(RequestURL, RequestMethod, ExtraRequestHeaders, MySettings.AWSAccessKeyId, MySettings.AWSSecretAccessKey);
                ExtraRequestHeaders.Add("Authorization", AuthorizationValue);

                Boolean RetBool;
                RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, ExtraRequestHeaders, "");

                ToolStripStatusLabelAction.Text = "";

                if (MyREST.ResponseStatusCode == 200)
                {
                    //the file exists.  Show the conflict form
                    MyFormConflictUpload = new FormConflictUpload();
                    MyFormConflictUpload.TextBoxRemoteBucketName.Text = TextBoxUploadBucketName.Text;
                    MyFormConflictUpload.TextBoxRemoteKeyName.Text = TextBoxUploadKeyName.Text;
                    MyFormConflictUpload.TextBoxRemoteSize.Text = Convert.ToDouble(MyREST.ResponseHeaders["Content-Length"]).ToString("N0") + " bytes";
                    MyFormConflictUpload.TextBoxRemoteAmazonDate.Text = Convert.ToString(Convert.ToDateTime(MyREST.ResponseHeaders["Last-Modified"]));
                    MyFormConflictUpload.TextBoxRemoteETag.Text = MyREST.ResponseHeaders["ETag"].Replace("\"", "");

                    System.IO.FileInfo LocalFileInfo = new System.IO.FileInfo(TextBoxUploadFileName.Text);
                    MyFormConflictUpload.TextBoxLocalFileName.Text = TextBoxUploadFileName.Text;
                    MyFormConflictUpload.TextBoxLocalSize.Text = LocalFileInfo.Length.ToString("N0") + " bytes";
                    MyFormConflictUpload.TextBoxLocalDate.Text = Convert.ToString(LocalFileInfo.LastWriteTime);

                    //kick off a process to calculate the etag.  A large file may take a long time to calculate.
                    MyFormConflictUpload.TextBoxLocalETag.Text = "calculating....";
                    System.Collections.Hashtable ETagHashTable = new System.Collections.Hashtable();
                    ETagHashTable.Add("LocalFileName", TextBoxUploadFileName.Text);
                    ETagBackgroundWorker.RunWorkerAsync(ETagHashTable);

                    //show the conflict form
                    System.Windows.Forms.DialogResult DialogResult;
                    DialogResult = MyFormConflictUpload.ShowDialog(this);

                    if (DialogResult == System.Windows.Forms.DialogResult.Yes)
                    {
                        //the use agreed to overwrite the file, do nothing
                    }
                    else
                    {
                        EnableDisableEnd();
                        MessageBox.Show("Upload skipped.", "Skipped", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return;
                    }
                }


                //abort the calculation of the etag if it is taking place
                if (MyCalculateHash.State == 1)
                {
                    MyCalculateHash.Abort();
                    //wait until the abort sequence is complete
                    do
                    {
                        System.Threading.Thread.Sleep(100);
                    } while (MyCalculateHash.State == 1);
                }


                //Start by calculating the MD5 hash of the file.  This will be sent to Amazon in the 
                //upload to make sure it arrives correctly.
                ToolStripStatusLabelAction.Text = "Calculating MD5 hash....";
                Application.DoEvents();

                //Run the hash calculation in a BackgroundWorker process.  Calculating the hash of a
                //large file will take a while.  Running the process in a BackgroundWorker will prevent
                //the form from locking up.

                //Use a hash table to pass parameters to the function in the BackgroundWorker.
                System.Collections.Hashtable CalculateHashHashTable = new System.Collections.Hashtable();
                CalculateHashHashTable.Add("LocalFileName", TextBoxUploadFileName.Text);

                CalculateHashBackgroundWorker.RunWorkerAsync(CalculateHashHashTable);
            }
            else
            {
                EnableDisableEnd();
                MessageBox.Show("The local file does not exist.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        private void CalculateHashBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            //Call the CalculateMD5FromFile function and set the result.  When the function is complete
            //the RunWorkerCompleted event will fire.  Use the LocalFileName value from the passed hash table.
            System.Collections.Hashtable CalculateHashHashTable = e.Argument as System.Collections.Hashtable;
            e.Result = MyCalculateHash.CalculateMD5FromFile(CalculateHashHashTable["LocalFileName"].ToString());
        }

        private void CalculateHashBackgroundWorker_RunWorkerCompleted(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
        {

            if (MyCalculateHash.ErrorNumber == 0)
            {
                ToolStripStatusLabelAction.Visible = false;
                ToolStripProgressBar.Visible = true;
                ToolStripStatusLabelBytes.Visible = true;
                ToolStripStatusLabelSpeed.Visible = true;
                ToolStripStatusLabelElapsed.Visible = true;
                ToolStripStatusLabelRemaining.Visible = true;

                Application.DoEvents();

                //Set the extra request headers to send with the upload
                Dictionary<String, String> ExtraRequestHeaders = new Dictionary<String, String>();

                if (TextBoxUploadContentType.Text != "")
                {
                    ExtraRequestHeaders.Add("Content-Type", TextBoxUploadContentType.Text);
                }

                if (CheckBoxUploadMakePublic.Checked == true)
                {
                    ExtraRequestHeaders.Add("x-amz-acl", "public-read");
                }

                //Use the MD5 hash that was calculated previously.
                ExtraRequestHeaders.Add("Content-MD5", e.Result.ToString());

                Properties.Settings MySettings = new Properties.Settings();

                String RequestURL;
                RequestURL = MyUpload.BuildS3RequestURL(true, "s3.amazonaws.com", TextBoxUploadBucketName.Text, TextBoxUploadKeyName.Text, "");

                String RequestMethod = "PUT";

                ExtraRequestHeaders.Add("x-amz-date", DateTime.UtcNow.ToString("r"));

                String AuthorizationValue;
                AuthorizationValue = MyUpload.GetS3AuthorizationValue(RequestURL, RequestMethod, ExtraRequestHeaders, MySettings.AWSAccessKeyId, MySettings.AWSSecretAccessKey);
                ExtraRequestHeaders.Add("Authorization", AuthorizationValue);

                //Create a hash table of of parameters to sent to the upload function.
                System.Collections.Hashtable UploadHashTable = new System.Collections.Hashtable();
                UploadHashTable.Add("RequestURL", RequestURL);
                UploadHashTable.Add("RequestMethod", RequestMethod);
                UploadHashTable.Add("ExtraRequestHeaders", ExtraRequestHeaders);
                UploadHashTable.Add("LocalFileName", TextBoxUploadFileName.Text);

                //Run the UploadFile call in a BackgroundWorker to prevent the Window from freezing.
                UploadBackgroundWorker.RunWorkerAsync(UploadHashTable);
            }
            else
            {
                EnableDisableEnd();
                MessageBox.Show(MyCalculateHash.ErrorDescription, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void UploadBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            //Run the UploadFile call.
            System.Collections.Hashtable UploadHashTable = e.Argument as System.Collections.Hashtable;

            e.Result = MyUpload.UploadFile(UploadHashTable["RequestURL"].ToString(), UploadHashTable["RequestMethod"].ToString(), UploadHashTable["ExtraRequestHeaders"] as Dictionary<String, String>, UploadHashTable["LocalFileName"].ToString());
        }

        private void UploadBackgroundWorker_RunWorkerCompleted(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
        {
            System.Diagnostics.Debug.Print("");
            System.Diagnostics.Debug.Print(MyUpload.LogData);
            System.Diagnostics.Debug.Print("");

            EnableDisableEnd();

            if (Convert.ToBoolean(e.Result) == true)
            {
                MessageBox.Show("Upload complete.", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                //Show the error message.
                String ResponseMessage;

                if (MyUpload.ResponseString == "")
                {
                    ResponseMessage = MyUpload.ErrorDescription;
                }
                else
                {
                    System.Xml.XmlDocument XmlDoc = new System.Xml.XmlDocument();
                    XmlDoc.LoadXml(MyUpload.ResponseString);

                    System.Xml.XmlNode XmlNode;
                    XmlNode = XmlDoc.SelectSingleNode("/Error/Message");

                    ResponseMessage = XmlNode.InnerText;
                }

                MessageBox.Show(ResponseMessage, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonAbort_Click(object sender, EventArgs e)
        {
            //Abort the CalculateHash call or Upload call depending on what is taking place.
            if (MyCalculateHash.State == 1)
            {
                MyCalculateHash.Abort();
            }
            else if (MyUpload.State == 1)
            {
                MyUpload.Abort();
            }
        }

        private void MyUpload_ProgressChangedEvent()
        {
            if (this.InvokeRequired == true)
            {
                this.BeginInvoke(new MethodInvoker(delegate() { MyUpload_ProgressChangedEvent(); }));
            }
            else
            {
                //Update the progress bar when the ProgressChangedEvent is fired.
                if (MyUpload.BytesTotal > 0)
                {
                    decimal MyDecimal = (Convert.ToDecimal(MyUpload.BytesTransfered) / Convert.ToDecimal(MyUpload.BytesTotal)) * 100;
                    ToolStripProgressBar.Value = Convert.ToInt32(MyDecimal);

                    SprightlySoftAWS.S3.Helper MyHelper = new SprightlySoftAWS.S3.Helper();
                    ToolStripStatusLabelBytes.Text = MyHelper.FormatByteSize(MyUpload.BytesTransfered) + " / " + MyHelper.FormatByteSize(MyUpload.BytesTotal);
                }
            }
        }

        private void TimerUpdateStatus_Tick(object source, System.Timers.ElapsedEventArgs e)
        {
            if (this.InvokeRequired == true)
            {
                this.BeginInvoke(new MethodInvoker(delegate() { TimerUpdateStatus_Tick(source, e); }));
            }
            else
            {
                //get the current speed, time elapsed and time remaining from the UploadEstimate class
                ToolStripStatusLabelSpeed.Text = MyUpload.CurrentKBpsSpeed.ToString("N0") + " KB/s";

                System.TimeSpan ElapsedTimeSpan;
                ElapsedTimeSpan = new System.TimeSpan(0, 0, Convert.ToInt32(MyUpload.SecondsElapsed));
                ToolStripStatusLabelElapsed.Text = ElapsedTimeSpan.ToString() + " elapsed";

                System.TimeSpan RemainingTimeSpan;
                RemainingTimeSpan = new System.TimeSpan(0, 0, Convert.ToInt32(MyUpload.SecondsRemaining));
                ToolStripStatusLabelRemaining.Text = RemainingTimeSpan.ToString() + " remaining";
            }
        }

        private void ETagBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            System.Collections.Hashtable CalculateHashHashTable = e.Argument as System.Collections.Hashtable;
            e.Result = MyCalculateHash.CalculateETagFromFile(CalculateHashHashTable["LocalFileName"].ToString());
        }

        private void ETagBackgroundWorker_RunWorkerCompleted(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
        {
            //show the etag in the conflict form
            MyFormConflictUpload.TextBoxLocalETag.Text = e.Result.ToString();
        }

    }
}
