﻿Public Class FormMain

    Dim AWSErrorRetries As Integer = 3

    Private Sub FormMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        TextBoxAWSAccessKeyId.Text = My.Settings.AWSAccessKeyId
        TextBoxAWSSecretAccessKey.Text = My.Settings.AWSSecretAccessKey
    End Sub

    Private Sub FormMain_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        My.Settings.AWSAccessKeyId = TextBoxAWSAccessKeyId.Text
        My.Settings.AWSSecretAccessKey = TextBoxAWSSecretAccessKey.Text
    End Sub

    Private Sub LinkLabelGetAmazonAccount_LinkClicked(ByVal sender As System.Object, ByVal e As System.Windows.Forms.LinkLabelLinkClickedEventArgs) Handles LinkLabelGetAmazonAccount.LinkClicked
        System.Diagnostics.Process.Start("http://aws.amazon.com/")
    End Sub

    Private Function FormatLogData(ByVal RequestURL As String, ByVal RequestMethod As String, ByVal RequestHeaders As Dictionary(Of String, String), ByVal ResponseStatusCode As Integer, ByVal ResponseStatusDescription As String, ByVal ResponseHeaders As Dictionary(Of String, String), ByVal ResponseString As String, ByVal ErrorNumber As Integer, ByVal ErrorDescription As String) As String

        Dim ReturnString As String = ""

        If ErrorNumber <> 0 Then
            ReturnString += "SprightlySoft ErrorDescription: " & ErrorDescription & vbCrLf
            ReturnString += "SprightlySoft ErrorNumber: " & ErrorNumber & vbCrLf
            ReturnString += vbCrLf
        End If

        If ResponseStatusCode <> 0 Then
            ReturnString += "Request URL: " & RequestURL & vbCrLf
            ReturnString += "Request Method: " & RequestMethod & vbCrLf

            For Each MyHeader As KeyValuePair(Of String, String) In RequestHeaders
                ReturnString += "Request Header: " & MyHeader.Key & ":" & MyHeader.Value & vbCrLf
            Next

            ReturnString += vbCrLf
            ReturnString += "Response Status Code: " & ResponseStatusCode & vbCrLf
            ReturnString += "Response Status Description: " & ResponseStatusDescription & vbCrLf

            For Each MyHeader As KeyValuePair(Of String, String) In ResponseHeaders
                ReturnString += "Response Header: " & MyHeader.Key & ":" & MyHeader.Value & vbCrLf
            Next

            If ResponseString <> "" Then
                ReturnString += vbCrLf

                Try
                    Dim XmlDoc As New System.Xml.XmlDocument
                    XmlDoc.LoadXml(ResponseString)

                    ReturnString += "Response XML: " & vbCrLf & ResponseString & vbCrLf

                    Dim XmlNode As System.Xml.XmlNode
                    XmlNode = XmlDoc.SelectSingleNode("/Error/Message")

                    If IsNothing(XmlNode) = False Then
                        ReturnString += vbCrLf
                        ReturnString += "Amazon Error Message: " & XmlNode.InnerText & vbCrLf
                    End If

                Catch ex As Exception
                    ReturnString += "Response String: " & vbCrLf & ResponseString & vbCrLf

                End Try

            End If

        End If

        Return ReturnString

    End Function

    Private Sub ButtonEC2DescribeImages_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonEC2DescribeImages.Click
        sender.Enabled = False

        'Amazon Elastic Compute Cloud (EC2), DescribeImages: http://docs.amazonwebservices.com/AWSEC2/latest/APIReference/ApiReference-query-DescribeImages.html

        Dim MyREST As New SprightlySoftAWS.REST

        Dim RequestURL As String
        RequestURL = "https://ec2.amazonaws.com/?Action=DescribeImages&Version=2010-08-31"
        RequestURL += "&AWSAccessKeyId=" & System.Uri.EscapeDataString(TextBoxAWSAccessKeyId.Text) & "&SignatureVersion=2&SignatureMethod=HmacSHA256&Timestamp=" & Uri.EscapeDataString(DateTime.UtcNow.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))
        RequestURL += "&Filter.1.Name=is-public&Filter.1.Value.1=true"
        RequestURL += "&Filter.2.Name=architecture&Filter.2.Value.1=x86_64"
        RequestURL += "&Filter.3.Name=platform&Filter.3.Value.1=windows"

        Dim RequestMethod As String
        RequestMethod = "GET"

        Dim SignatureValue As String
        SignatureValue = MyREST.GetSignatureVersion2Value(RequestURL, RequestMethod, "", TextBoxAWSSecretAccessKey.Text)

        RequestURL += "&Signature=" & System.Uri.EscapeDataString(SignatureValue)

        Dim RetBool As Boolean
        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, Nothing)

        Debug.Print("")
        Debug.Print(MyREST.LogData)
        Debug.Print("")

        Dim ResponseMessage As String = ""

        If RetBool = True Then
            Dim MyXmlDocument As System.Xml.XmlDocument
            Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim MyXmlNode As System.Xml.XmlNode
            Dim MyXmlNodeList As System.Xml.XmlNodeList

            MyXmlDocument = New System.Xml.XmlDocument
            MyXmlDocument.LoadXml(MyREST.ResponseString)

            MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
            MyXmlNamespaceManager.AddNamespace("amz", "http://ec2.amazonaws.com/doc/2010-08-31/")

            MyXmlNodeList = MyXmlDocument.SelectNodes("amz:DescribeImagesResponse/amz:imagesSet/amz:item", MyXmlNamespaceManager)

            If MyXmlNodeList.Count = 0 Then
                ResponseMessage = "No images found."
            Else
                For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:imageId", MyXmlNamespaceManager)
                    ResponseMessage += "imageId = " & MyXmlNode.InnerText

                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:name", MyXmlNamespaceManager)
                    If IsNothing(MyXmlNode) = False Then
                        ResponseMessage += "   Name = " & MyXmlNode.InnerText
                    End If

                    ResponseMessage += vbCrLf
                Next
            End If

            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)

        Else
            ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub

    Private Sub ButtonElasticMapReduceDescribeJobFlows_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonElasticMapReduceDescribeJobFlows.Click
        sender.Enabled = False

        'Amazon Elastic MapReduce, DescribeJobFlows: http://docs.amazonwebservices.com/ElasticMapReduce/latest/API/API_DescribeJobFlows.html

        Dim MyREST As New SprightlySoftAWS.REST

        'the CreatedBefore parameter is set so you can see how to format the value
        Dim CreatedBefore As Date
        CreatedBefore = Now

        Dim RequestURL As String
        RequestURL = "https://elasticmapreduce.amazonaws.com/?Operation=DescribeJobFlows&Version=2009-03-31"
        RequestURL += "&AWSAccessKeyId=" & System.Uri.EscapeDataString(TextBoxAWSAccessKeyId.Text) & "&SignatureVersion=2&SignatureMethod=HmacSHA256&Timestamp=" & Uri.EscapeDataString(DateTime.UtcNow.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))
        RequestURL += "&CreatedBefore=" & Uri.EscapeDataString(CreatedBefore.ToUniversalTime.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))

        Dim RequestMethod As String
        RequestMethod = "GET"

        Dim SignatureValue As String
        SignatureValue = MyREST.GetSignatureVersion2Value(RequestURL, RequestMethod, "", TextBoxAWSSecretAccessKey.Text)

        RequestURL += "&Signature=" & System.Uri.EscapeDataString(SignatureValue)

        Dim RetBool As Boolean
        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, Nothing)

        Debug.Print("")
        Debug.Print(MyREST.LogData)
        Debug.Print("")

        Dim ResponseMessage As String = ""

        If RetBool = True Then
            Dim MyXmlDocument As System.Xml.XmlDocument
            Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim MyXmlNode As System.Xml.XmlNode
            Dim MyXmlNodeList As System.Xml.XmlNodeList

            MyXmlDocument = New System.Xml.XmlDocument
            MyXmlDocument.LoadXml(MyREST.ResponseString)

            MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
            MyXmlNamespaceManager.AddNamespace("amz", "http://elasticmapreduce.amazonaws.com/doc/2009-03-31")

            MyXmlNodeList = MyXmlDocument.SelectNodes("amz:DescribeJobFlowsResponse/amz:DescribeJobFlowsResult/amz:JobFlows/amz:member", MyXmlNamespaceManager)

            If MyXmlNodeList.Count = 0 Then
                ResponseMessage = "No job flows exist."
            Else

                For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:Name", MyXmlNamespaceManager)
                    ResponseMessage += "Name = " & MyXmlNode.InnerText

                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:ExecutionStatusDetail/amz:CreationDateTime", MyXmlNamespaceManager)
                    ResponseMessage += "   CreationDateTime = " & MyXmlNode.InnerText

                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:ExecutionStatusDetail/amz:State", MyXmlNamespaceManager)
                    ResponseMessage += "   State = " & MyXmlNode.InnerText

                    ResponseMessage += vbCrLf
                Next
            End If

            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)

        Else
            ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub

    Private Sub ButtonAutoScalingDescribeAutoScalingGroups_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonAutoScalingDescribeAutoScalingGroups.Click
        sender.Enabled = False

        'Auto Scaling, DescribeAutoScalingGroups: http://docs.amazonwebservices.com/AutoScaling/latest/DeveloperGuide/API_DescribeAutoScalingGroups.html

        Dim MyREST As New SprightlySoftAWS.REST

        Dim RequestURL As String
        RequestURL = "https://autoscaling.amazonaws.com/?Action=DescribeAutoScalingGroups&Version=2009-05-15"
        RequestURL += "&AWSAccessKeyId=" & System.Uri.EscapeDataString(TextBoxAWSAccessKeyId.Text) & "&SignatureVersion=2&SignatureMethod=HmacSHA1&Timestamp=" & Uri.EscapeDataString(DateTime.UtcNow.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))
        RequestURL += "&AutoScalingGroupNames.member.1=webtier"

        Dim RequestMethod As String
        RequestMethod = "GET"

        Dim SignatureValue As String
        SignatureValue = MyREST.GetSignatureVersion2Value(RequestURL, RequestMethod, "", TextBoxAWSSecretAccessKey.Text)

        RequestURL += "&Signature=" & System.Uri.EscapeDataString(SignatureValue)

        Dim RetBool As Boolean
        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, Nothing)

        Debug.Print("")
        Debug.Print(MyREST.LogData)
        Debug.Print("")

        Dim ResponseMessage As String = ""

        If RetBool = True Then
            Dim MyXmlDocument As System.Xml.XmlDocument
            Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim MyXmlNode As System.Xml.XmlNode
            Dim MyXmlNodeList As System.Xml.XmlNodeList

            MyXmlDocument = New System.Xml.XmlDocument
            MyXmlDocument.LoadXml(MyREST.ResponseString)

            MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
            MyXmlNamespaceManager.AddNamespace("amz", "http://autoscaling.amazonaws.com/doc/2009-05-15/")

            MyXmlNodeList = MyXmlDocument.SelectNodes("amz:DescribeAutoScalingGroupsResponse/amz:DescribeAutoScalingGroupsResult/amz:AutoScalingGroups/amz:member", MyXmlNamespaceManager)

            If MyXmlNodeList.Count = 0 Then
                ResponseMessage = "No auto scaling groups exist."
            Else
                For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:AutoScalingGroupName", MyXmlNamespaceManager)
                    ResponseMessage += "AutoScalingGroupName = " & MyXmlNode.InnerText

                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:CreatedTime", MyXmlNamespaceManager)
                    ResponseMessage += "   CreatedTime = " & MyXmlNode.InnerText

                    ResponseMessage += vbCrLf
                Next
            End If

            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)

        Else
            ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub

    Private Sub ButtonCloudFrontGETDistributionList_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonCloudFrontGETDistributionList.Click
        sender.Enabled = False

        'CloudFront, GET Distribution List: http://docs.amazonwebservices.com/AmazonCloudFront/latest/APIReference/ListDistributions.html

        Dim Marker As String = ""
        Dim MaxItems As Integer = 100  '100 is the maximum number of distributions you want in the response body
        Dim IsTruncated As Boolean = False

        Dim RetBool As Boolean
        Dim MyXmlDocument As System.Xml.XmlDocument
        Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
        Dim MyXmlNode As System.Xml.XmlNode
        Dim MyXmlNodeList As System.Xml.XmlNodeList
        Dim ResponseMessage As String = ""

        Dim MyREST As New SprightlySoftAWS.REST
        Dim RequestURL As String
        Dim ExtraRequestHeaders As Dictionary(Of String, String)
        Dim AmzDate As String
        Dim AuthorizationValue As String

        'put the following code in a loop as we may be listing many pages
        Do
            RequestURL = "https://cloudfront.amazonaws.com/2010-11-01/distribution?Marker=" & System.Uri.EscapeDataString(Marker) & "&MaxItems=" & System.Uri.EscapeDataString(MaxItems)

            ExtraRequestHeaders = New Dictionary(Of String, String)

            AmzDate = DateTime.UtcNow.ToString("r")
            ExtraRequestHeaders.Add("x-amz-date", AmzDate)

            AuthorizationValue = MyREST.GetCloudFrontAuthorizationValue(TextBoxAWSAccessKeyId.Text, TextBoxAWSSecretAccessKey.Text, AmzDate)
            ExtraRequestHeaders.Add("Authorization", AuthorizationValue)

            RetBool = MyREST.MakeRequest(RequestURL, "GET", ExtraRequestHeaders, "")

            Debug.Print("")
            Debug.Print(MyREST.LogData)
            Debug.Print("")

            If RetBool = True Then

                MyXmlDocument = New System.Xml.XmlDocument
                MyXmlDocument.LoadXml(MyREST.ResponseString)

                MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
                MyXmlNamespaceManager.AddNamespace("amz", "http://cloudfront.amazonaws.com/doc/2010-11-01/")

                MyXmlNode = MyXmlDocument.SelectSingleNode("amz:DistributionList/amz:NextMarker", MyXmlNamespaceManager)
                If IsNothing(MyXmlNode) = False Then
                    'set the Marker to the NextMarker value
                    Marker = MyXmlNode.InnerText
                End If

                MyXmlNode = MyXmlDocument.SelectSingleNode("amz:DistributionList/amz:IsTruncated", MyXmlNamespaceManager)
                IsTruncated = MyXmlNode.InnerText

                MyXmlNodeList = MyXmlDocument.SelectNodes("amz:DistributionList/amz:DistributionSummary", MyXmlNamespaceManager)

                If MyXmlNodeList.Count = 0 Then
                    ResponseMessage = "No distributions exist."
                Else
                    For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                        MyXmlNode = ItemXmlNode.SelectSingleNode("amz:Id", MyXmlNamespaceManager)
                        ResponseMessage = ResponseMessage & "Id = " & MyXmlNode.InnerText

                        MyXmlNode = ItemXmlNode.SelectSingleNode("amz:Status", MyXmlNamespaceManager)
                        ResponseMessage = ResponseMessage & "     Status = " & MyXmlNode.InnerText

                        MyXmlNode = ItemXmlNode.SelectSingleNode("amz:LastModifiedTime", MyXmlNamespaceManager)
                        ResponseMessage = ResponseMessage & "     LastModifiedTime = " & CDate(MyXmlNode.InnerText)

                        MyXmlNode = ItemXmlNode.SelectSingleNode("amz:DomainName", MyXmlNamespaceManager)
                        ResponseMessage = ResponseMessage & "     DomainName = " & MyXmlNode.InnerText

                        MyXmlNode = ItemXmlNode.SelectSingleNode("amz:Origin", MyXmlNamespaceManager)
                        ResponseMessage = ResponseMessage & "     Origin = " & MyXmlNode.InnerText

                        MyXmlNode = ItemXmlNode.SelectSingleNode("amz:CNAME", MyXmlNamespaceManager)
                        If IsNothing(MyXmlNode) = False Then
                            ResponseMessage = ResponseMessage & "     CNAME = " & MyXmlNode.InnerText
                        End If

                        MyXmlNode = ItemXmlNode.SelectSingleNode("amz:Comment", MyXmlNamespaceManager)
                        If IsNothing(MyXmlNode) = False Then
                            ResponseMessage = ResponseMessage & "     Comment = " & MyXmlNode.InnerText
                        End If

                        MyXmlNode = ItemXmlNode.SelectSingleNode("amz:Enabled", MyXmlNamespaceManager)
                        ResponseMessage = ResponseMessage & "     Enabled = " & MyXmlNode.InnerText

                        MyXmlNode = ItemXmlNode.SelectSingleNode("amz:TrustedSigners/amz:Self", MyXmlNamespaceManager)
                        If IsNothing(MyXmlNode) = False Then
                            ResponseMessage = ResponseMessage & "     TrustedSigners Self = " & MyXmlNode.InnerText
                        End If

                        MyXmlNode = ItemXmlNode.SelectSingleNode("amz:TrustedSigners/amz:AwsAccountNumber", MyXmlNamespaceManager)
                        If IsNothing(MyXmlNode) = False Then
                            ResponseMessage = ResponseMessage & "     TrustedSigners AwsAccountNumber = " & MyXmlNode.InnerText
                        End If

                        ResponseMessage = ResponseMessage & vbCrLf
                    Next
                End If

            Else
                ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

                DialogOutput.Text = "Error"
                DialogOutput.TextBoxOutput.Text = ResponseMessage
                DialogOutput.ShowDialog(Me)

                Exit Do
            End If


            If IsTruncated = False Then
                'if there are no more pages to get, show the output and exit
                DialogOutput.Text = "Success"
                DialogOutput.TextBoxOutput.Text = ResponseMessage
                DialogOutput.ShowDialog(Me)

                Exit Do
            End If

        Loop

        sender.Enabled = True
    End Sub

    Private Sub ButtonSimpleDBListDomains_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonSimpleDBListDomains.Click
        sender.Enabled = False

        'Amazon SimpleDB, ListDomains: http://docs.amazonwebservices.com/AmazonSimpleDB/latest/DeveloperGuide/SDB_API_ListDomains.html

        Dim MyREST As New SprightlySoftAWS.REST

        Dim RequestURL As String
        RequestURL = "https://sdb.amazonaws.com/"

        'You can post parameters instead of putting them in the query string.  This is useful if you want to avoid creating a URL that is too long when sending many parameters.
        Dim PostData As String
        PostData = "Action=ListDomains&Version=2009-04-15&AWSAccessKeyId=" & System.Uri.EscapeDataString(TextBoxAWSAccessKeyId.Text) & "&SignatureVersion=2&SignatureMethod=HmacSHA256&Timestamp=" & Uri.EscapeDataString(DateTime.UtcNow.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))

        Dim RequestMethod As String
        RequestMethod = "POST"

        Dim SignatureValue As String
        SignatureValue = MyREST.GetSignatureVersion2Value(RequestURL, RequestMethod, PostData, TextBoxAWSSecretAccessKey.Text)

        PostData += "&Signature=" & System.Uri.EscapeDataString(SignatureValue)

        Dim ExtraRequestHeaders As New Dictionary(Of String, String)
        ExtraRequestHeaders.Add("Content-Type", "application/x-www-form-urlencoded; charset=utf-8")

        Dim RetBool As Boolean
        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, ExtraRequestHeaders, PostData)

        Debug.Print("")
        Debug.Print(MyREST.LogData)
        Debug.Print("")

        Dim ResponseMessage As String = ""

        If RetBool = True Then
            Dim MyXmlDocument As System.Xml.XmlDocument
            Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim MyXmlNodeList As System.Xml.XmlNodeList

            MyXmlDocument = New System.Xml.XmlDocument
            MyXmlDocument.LoadXml(MyREST.ResponseString)

            MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
            MyXmlNamespaceManager.AddNamespace("amz", "http://autoscaling.amazonaws.com/doc/2009-05-15/")

            MyXmlNodeList = MyXmlDocument.SelectNodes("amz:ListDomainsResponse/amz:ListDomainsResult/amz:DomainName", MyXmlNamespaceManager)

            If MyXmlNodeList.Count = 0 Then
                ResponseMessage = "No SimpleDB domains exist."
            Else
                For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                    ResponseMessage += "DomainName = " & ItemXmlNode.InnerText

                    ResponseMessage += vbCrLf
                Next
            End If

            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)

        Else
            ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub

    Private Sub ButtonRDSDescribeDBInstances_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonRDSDescribeDBInstances.Click
        sender.Enabled = False

        'Amazon Relational Database Service (RDS), DescribeDBInstances: http://docs.amazonwebservices.com/AmazonRDS/latest/APIReference/APIDescribeDBInstances.html

        Dim MyREST As New SprightlySoftAWS.REST

        Dim RequestURL As String
        RequestURL = "https://rds.amazonaws.com/?Action=DescribeDBInstances&Version=2010-07-28"
        RequestURL += "&AWSAccessKeyId=" & System.Uri.EscapeDataString(TextBoxAWSAccessKeyId.Text) & "&SignatureVersion=2&SignatureMethod=HmacSHA256&Timestamp=" & Uri.EscapeDataString(DateTime.UtcNow.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))

        Dim RequestMethod As String
        RequestMethod = "GET"

        Dim SignatureValue As String
        SignatureValue = MyREST.GetSignatureVersion2Value(RequestURL, RequestMethod, "", TextBoxAWSSecretAccessKey.Text)

        RequestURL += "&Signature=" & System.Uri.EscapeDataString(SignatureValue)

        Dim RetBool As Boolean
        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, Nothing)

        Debug.Print("")
        Debug.Print(MyREST.LogData)
        Debug.Print("")

        Dim ResponseMessage As String = ""

        If RetBool = True Then
            Dim MyXmlDocument As System.Xml.XmlDocument
            Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim MyXmlNode As System.Xml.XmlNode
            Dim MyXmlNodeList As System.Xml.XmlNodeList

            MyXmlDocument = New System.Xml.XmlDocument
            MyXmlDocument.LoadXml(MyREST.ResponseString)

            MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
            MyXmlNamespaceManager.AddNamespace("amz", "http://rds.amazonaws.com/doc/2010-07-28/")

            MyXmlNodeList = MyXmlDocument.SelectNodes("amz:DescribeDBInstancesResponse/amz:DescribeDBInstancesResult/amz:DBInstances/amz:DBInstance", MyXmlNamespaceManager)

            If MyXmlNodeList.Count = 0 Then
                ResponseMessage = "No instances exist."
            Else
                For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:DBInstanceIdentifier", MyXmlNamespaceManager)
                    ResponseMessage += "DBInstanceIdentifier = " & MyXmlNode.InnerText

                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:DBInstanceStatus", MyXmlNamespaceManager)
                    ResponseMessage += "   DBInstanceStatus = " & MyXmlNode.InnerText

                    ResponseMessage += vbCrLf
                Next
            End If

            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)

        Else
            ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub

    Private Sub ButtonFWSListAllFulfillmentOrders_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonFWSListAllFulfillmentOrders.Click
        sender.Enabled = False

        'Amazon Fulfillment Web Service (FWS), ListAllFulfillmentOrders: http://docs.amazonwebservices.com/fws/latest/APIReference/ListAllFulfillmentOrders.html

        Dim MyREST As New SprightlySoftAWS.REST

        Dim RequestURL As String
        RequestURL = "https://fba-outbound.amazonaws.com/?Action=ListAllFulfillmentOrders&Version=2007-08-02"
        RequestURL += "&AWSAccessKeyId=" & System.Uri.EscapeDataString(TextBoxAWSAccessKeyId.Text) & "&Timestamp=" & Uri.EscapeDataString(DateTime.UtcNow.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))
        RequestURL += "&NumberOfResultsRequested=3"

        Dim RequestMethod As String
        RequestMethod = "GET"

        Dim SignatureValue As String
        SignatureValue = MyREST.GetFulfillmentWebServiceSignatureValue(RequestURL, TextBoxAWSSecretAccessKey.Text)

        RequestURL += "&Signature=" & System.Uri.EscapeDataString(SignatureValue)

        Dim RetBool As Boolean
        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, Nothing)

        Debug.Print("")
        Debug.Print(MyREST.LogData)
        Debug.Print("")

        Dim ResponseMessage As String = ""

        If RetBool = True Then
            Dim EnvelopeXmlDocument As System.Xml.XmlDocument
            Dim EnvelopeXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim EnvelopeXmlNode As System.Xml.XmlNode
            Dim MyXmlDocument As System.Xml.XmlDocument
            Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim MyXmlNode As System.Xml.XmlNode
            Dim MyXmlNodeList As System.Xml.XmlNodeList

            EnvelopeXmlDocument = New System.Xml.XmlDocument
            EnvelopeXmlDocument.LoadXml(MyREST.ResponseString)

            EnvelopeXmlNamespaceManager = New System.Xml.XmlNamespaceManager(EnvelopeXmlDocument.NameTable)
            EnvelopeXmlNamespaceManager.AddNamespace("env", "http://schemas.xmlsoap.org/soap/envelope/")

            EnvelopeXmlNode = EnvelopeXmlDocument.SelectSingleNode("env:Envelope/env:Body", EnvelopeXmlNamespaceManager)

            MyXmlDocument = New System.Xml.XmlDocument
            MyXmlDocument.LoadXml(EnvelopeXmlNode.InnerXml)

            MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
            MyXmlNamespaceManager.AddNamespace("amz", "http://fba-outbound.amazonaws.com/doc/2007-08-02/")

            MyXmlNodeList = MyXmlDocument.SelectNodes("amz:ListAllFulfillmentOrdersResponse/amz:ListAllFulfillmentOrdersResult/amz:FulfillmentOrder", MyXmlNamespaceManager)

            If MyXmlNodeList.Count = 0 Then
                ResponseMessage = "No fulfillment orders exist."
            Else
                For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:MerchantFulfillmentOrderId", MyXmlNamespaceManager)
                    ResponseMessage += "MerchantFulfillmentOrderId = " & MyXmlNode.InnerText

                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:ReceivedDateTime", MyXmlNamespaceManager)
                    ResponseMessage += "   ReceivedDateTime = " & MyXmlNode.InnerText

                    ResponseMessage += vbCrLf
                Next
            End If

            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)

        Else
            ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub

    Private Sub ButtonIAMListUsers_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonIAMListUsers.Click
        sender.Enabled = False

        'AWS Identity and Access Management (IAM), ListUsers: http://docs.amazonwebservices.com/IAM/latest/APIReference/API_ListUsers.html

        Dim Marker As String = ""
        Dim MaxItems As Integer = 100
        Dim PathPrefix As String = ""
        Dim IsTruncated As Boolean = False

        Dim RetBool As Boolean
        Dim MyXmlDocument As System.Xml.XmlDocument
        Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
        Dim MyXmlNode As System.Xml.XmlNode
        Dim MyXmlNodeList As System.Xml.XmlNodeList
        Dim ResponseMessage As String = ""

        Dim MyREST As New SprightlySoftAWS.REST
        Dim RequestURL As String
        Dim RequestMethod As String

        'put the following code in a loop as we may be listing many pages
        Do
            RequestURL = "https://iam.amazonaws.com/?Action=ListUsers&Version=2010-05-08&MaxItems=" & System.Uri.EscapeDataString(MaxItems)
            RequestURL += "&AWSAccessKeyId=" & System.Uri.EscapeDataString(TextBoxAWSAccessKeyId.Text) & "&SignatureVersion=2&SignatureMethod=HmacSHA256&Timestamp=" & Uri.EscapeDataString(DateTime.UtcNow.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))
            If Marker <> "" Then
                RequestURL += "&Marker=" & System.Uri.EscapeDataString(Marker)
            End If
            If PathPrefix <> "" Then
                RequestURL += "&PathPrefix=" & System.Uri.EscapeDataString(PathPrefix)
            End If

            RequestMethod = "GET"

            Dim SignatureValue As String
            SignatureValue = MyREST.GetSignatureVersion2Value(RequestURL, RequestMethod, "", TextBoxAWSSecretAccessKey.Text)

            RequestURL += "&Signature=" & System.Uri.EscapeDataString(SignatureValue)

            RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, Nothing)

            Debug.Print("")
            Debug.Print(MyREST.LogData)
            Debug.Print("")

            If RetBool = True Then

                MyXmlDocument = New System.Xml.XmlDocument
                MyXmlDocument.LoadXml(MyREST.ResponseString)

                MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
                MyXmlNamespaceManager.AddNamespace("amz", "https://iam.amazonaws.com/doc/2010-05-08/")

                MyXmlNode = MyXmlDocument.SelectSingleNode("amz:ListUsersResponse/amz:ListUsersResult/amz:IsTruncated", MyXmlNamespaceManager)
                IsTruncated = MyXmlNode.InnerText

                MyXmlNodeList = MyXmlDocument.SelectNodes("amz:ListUsersResponse/amz:ListUsersResult/amz:Users/amz:member", MyXmlNamespaceManager)

                If MyXmlNodeList.Count = 0 Then
                    ResponseMessage = "No users exist."
                Else
                    For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                        MyXmlNode = ItemXmlNode.SelectSingleNode("amz:UserName", MyXmlNamespaceManager)
                        ResponseMessage = ResponseMessage & "UserName = " & MyXmlNode.InnerText

                        MyXmlNode = ItemXmlNode.SelectSingleNode("amz:UserId", MyXmlNamespaceManager)
                        ResponseMessage = ResponseMessage & "     UserId = " & MyXmlNode.InnerText

                        ResponseMessage = ResponseMessage & vbCrLf
                    Next
                End If

            Else
                ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

                DialogOutput.Text = "Error"
                DialogOutput.TextBoxOutput.Text = ResponseMessage
                DialogOutput.ShowDialog(Me)

                Exit Do
            End If


            If IsTruncated = False Then
                'if there are no more pages to get, show the output and exit
                DialogOutput.Text = "Success"
                DialogOutput.TextBoxOutput.Text = ResponseMessage
                DialogOutput.ShowDialog(Me)

                Exit Do
            End If

        Loop

        sender.Enabled = True
    End Sub

    Private Sub ButtonSQSListQueues_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonSQSListQueues.Click
        sender.Enabled = False

        'Amazon Simple Queue Service (SQS), ListQueues: http://docs.amazonwebservices.com/AWSSimpleQueueService/latest/APIReference/Query_QueryListQueues.html

        Dim MyREST As New SprightlySoftAWS.REST

        Dim RequestURL As String
        RequestURL = "https://sqs.us-east-1.amazonaws.com/?Action=ListQueues&Version=2009-02-01"
        RequestURL += "&AWSAccessKeyId=" & System.Uri.EscapeDataString(TextBoxAWSAccessKeyId.Text) & "&SignatureVersion=2&SignatureMethod=HmacSHA256&Timestamp=" & Uri.EscapeDataString(DateTime.UtcNow.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))

        Dim RequestMethod As String
        RequestMethod = "GET"

        Dim SignatureValue As String
        SignatureValue = MyREST.GetSignatureVersion2Value(RequestURL, RequestMethod, "", TextBoxAWSSecretAccessKey.Text)

        RequestURL += "&Signature=" & System.Uri.EscapeDataString(SignatureValue)

        Dim RetBool As Boolean
        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, Nothing)

        Debug.Print("")
        Debug.Print(MyREST.LogData)
        Debug.Print("")

        Dim ResponseMessage As String = ""

        If RetBool = True Then
            Dim MyXmlDocument As System.Xml.XmlDocument
            Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim MyXmlNodeList As System.Xml.XmlNodeList

            MyXmlDocument = New System.Xml.XmlDocument
            MyXmlDocument.LoadXml(MyREST.ResponseString)

            MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
            MyXmlNamespaceManager.AddNamespace("amz", "http://queue.amazonaws.com/doc/2009-02-01/")

            MyXmlNodeList = MyXmlDocument.SelectNodes("amz:ListQueuesResponse/amz:ListQueuesResult/amz:QueueUrl", MyXmlNamespaceManager)

            If MyXmlNodeList.Count = 0 Then
                ResponseMessage = "No queues exist."
            Else
                For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                    ResponseMessage += "QueueUrl = " & ItemXmlNode.InnerText
                    ResponseMessage += vbCrLf
                Next
            End If

            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)

        Else
            ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub

    Private Sub ButtonSNSListTopics_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonSNSListTopics.Click
        sender.Enabled = False

        'Amazon Simple Notification Service (SNS), ListTopics: http://docs.amazonwebservices.com/sns/latest/api/API_ListTopics.html

        Dim MyREST As New SprightlySoftAWS.REST

        Dim RequestURL As String
        RequestURL = "https://sns.us-east-1.amazonaws.com/?Action=ListTopics&Version=2010-03-31"
        RequestURL += "&AWSAccessKeyId=" & System.Uri.EscapeDataString(TextBoxAWSAccessKeyId.Text) & "&SignatureVersion=2&SignatureMethod=HmacSHA256&Timestamp=" & Uri.EscapeDataString(DateTime.UtcNow.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))

        Dim RequestMethod As String
        RequestMethod = "GET"

        Dim SignatureValue As String
        SignatureValue = MyREST.GetSignatureVersion2Value(RequestURL, RequestMethod, "", TextBoxAWSSecretAccessKey.Text)

        RequestURL += "&Signature=" & System.Uri.EscapeDataString(SignatureValue)

        Dim RetBool As Boolean
        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, Nothing)

        Debug.Print("")
        Debug.Print(MyREST.LogData)
        Debug.Print("")

        Dim ResponseMessage As String = ""

        If RetBool = True Then
            Dim MyXmlDocument As System.Xml.XmlDocument
            Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim MyXmlNode As System.Xml.XmlNode
            Dim MyXmlNodeList As System.Xml.XmlNodeList

            MyXmlDocument = New System.Xml.XmlDocument
            MyXmlDocument.LoadXml(MyREST.ResponseString)

            MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
            MyXmlNamespaceManager.AddNamespace("amz", "http://sns.amazonaws.com/doc/2010-03-31/")

            MyXmlNodeList = MyXmlDocument.SelectNodes("amz:ListTopicsResponse/amz:ListTopicsResult/amz:Topics/amz:member", MyXmlNamespaceManager)

            If MyXmlNodeList.Count = 0 Then
                ResponseMessage = "No topics exist."
            Else

                For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:TopicArn", MyXmlNamespaceManager)
                    ResponseMessage += "TopicArn = " & MyXmlNode.InnerText

                    ResponseMessage += vbCrLf
                Next
            End If

            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)

        Else
            ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub

    Private Sub ButtonCloudWatchListMetrics_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonCloudWatchListMetrics.Click
        sender.Enabled = False

        'Amazon CloudWatch, ListMetrics: http://docs.amazonwebservices.com/AmazonCloudWatch/latest/DeveloperGuide/API-ListMetrics.html

        Dim MyREST As New SprightlySoftAWS.REST

        Dim RequestURL As String
        RequestURL = "https://monitoring.amazonaws.com/?Action=ListMetrics&Version=2009-05-15"
        RequestURL += "&AWSAccessKeyId=" & System.Uri.EscapeDataString(TextBoxAWSAccessKeyId.Text) & "&SignatureVersion=2&SignatureMethod=HmacSHA256&Timestamp=" & Uri.EscapeDataString(DateTime.UtcNow.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))

        Dim RequestMethod As String
        RequestMethod = "GET"

        Dim SignatureValue As String
        SignatureValue = MyREST.GetSignatureVersion2Value(RequestURL, RequestMethod, "", TextBoxAWSSecretAccessKey.Text)

        RequestURL += "&Signature=" & System.Uri.EscapeDataString(SignatureValue)

        Dim RetBool As Boolean
        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, Nothing)

        Debug.Print("")
        Debug.Print(MyREST.LogData)
        Debug.Print("")

        Dim ResponseMessage As String = ""

        If RetBool = True Then
            Dim MyXmlDocument As System.Xml.XmlDocument
            Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim MyXmlNode As System.Xml.XmlNode
            Dim MyXmlNodeList As System.Xml.XmlNodeList

            MyXmlDocument = New System.Xml.XmlDocument
            MyXmlDocument.LoadXml(MyREST.ResponseString)

            MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
            MyXmlNamespaceManager.AddNamespace("amz", "http://monitoring.amazonaws.com/doc/2009-05-15/")

            MyXmlNodeList = MyXmlDocument.SelectNodes("amz:ListMetricsResponse/amz:ListMetricsResult/amz:Metrics/amz:member", MyXmlNamespaceManager)

            If MyXmlNodeList.Count = 0 Then
                ResponseMessage = "No metrics exist."
            Else

                For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:MeasureName", MyXmlNamespaceManager)
                    ResponseMessage += "MeasureName = " & MyXmlNode.InnerText

                    ResponseMessage += vbCrLf
                Next
            End If

            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)

        Else
            ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub

    Private Sub ButtonVPCDescribeVpcs_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonVPCDescribeVpcs.Click
        sender.Enabled = False

        'Amazon Virtual Private Cloud (VPC), DescribeVpcs: http://docs.amazonwebservices.com/AmazonVPC/latest/APIReference/ApiReference-query-DescribeVpcs.html

        Dim MyREST As New SprightlySoftAWS.REST

        Dim RequestURL As String
        RequestURL = "https://ec2.amazonaws.com/?Action=DescribeVpcs&Version=2010-08-31"
        RequestURL += "&AWSAccessKeyId=" & System.Uri.EscapeDataString(TextBoxAWSAccessKeyId.Text) & "&SignatureVersion=2&SignatureMethod=HmacSHA256&Timestamp=" & Uri.EscapeDataString(DateTime.UtcNow.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))

        Dim RequestMethod As String
        RequestMethod = "GET"

        Dim SignatureValue As String
        SignatureValue = MyREST.GetSignatureVersion2Value(RequestURL, RequestMethod, "", TextBoxAWSSecretAccessKey.Text)

        RequestURL += "&Signature=" & System.Uri.EscapeDataString(SignatureValue)

        Dim RetBool As Boolean
        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, Nothing)

        Debug.Print("")
        Debug.Print(MyREST.LogData)
        Debug.Print("")

        Dim ResponseMessage As String = ""

        If RetBool = True Then
            Dim MyXmlDocument As System.Xml.XmlDocument
            Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim MyXmlNode As System.Xml.XmlNode
            Dim MyXmlNodeList As System.Xml.XmlNodeList

            MyXmlDocument = New System.Xml.XmlDocument
            MyXmlDocument.LoadXml(MyREST.ResponseString)

            MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
            MyXmlNamespaceManager.AddNamespace("amz", "http://ec2.amazonaws.com/doc/2010-08-31/")

            MyXmlNodeList = MyXmlDocument.SelectNodes("amz:DescribeVpcsResponse/amz:vpcSet/amz:item", MyXmlNamespaceManager)

            If MyXmlNodeList.Count = 0 Then
                ResponseMessage = "No VPCs exist."
            Else

                For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:vpcId", MyXmlNamespaceManager)
                    ResponseMessage += "vpcId = " & MyXmlNode.InnerText

                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:state", MyXmlNamespaceManager)
                    ResponseMessage += "   state = " & MyXmlNode.InnerText

                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:cidrBlock", MyXmlNamespaceManager)
                    ResponseMessage += "   cidrBlock = " & MyXmlNode.InnerText

                    ResponseMessage += vbCrLf
                Next
            End If

            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)

        Else
            ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub

    Private Sub ButtonElasticLoadBalancingDescribeLoadBalancers_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonElasticLoadBalancingDescribeLoadBalancers.Click
        sender.Enabled = False

        'Elastic Load Balancing, DescribeLoadBalancers: http://docs.amazonwebservices.com/ElasticLoadBalancing/latest/DeveloperGuide/API_DescribeLoadBalancers.html

        Dim MyREST As New SprightlySoftAWS.REST

        Dim RequestURL As String
        RequestURL = "https://elasticloadbalancing.amazonaws.com/?Action=DescribeLoadBalancers&Version=2010-07-01"
        RequestURL += "&AWSAccessKeyId=" & System.Uri.EscapeDataString(TextBoxAWSAccessKeyId.Text) & "&SignatureVersion=2&SignatureMethod=HmacSHA1&Timestamp=" & Uri.EscapeDataString(DateTime.UtcNow.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))

        Dim RequestMethod As String
        RequestMethod = "GET"

        Dim SignatureValue As String
        SignatureValue = MyREST.GetSignatureVersion2Value(RequestURL, RequestMethod, "", TextBoxAWSSecretAccessKey.Text)

        RequestURL += "&Signature=" & System.Uri.EscapeDataString(SignatureValue)

        Dim RetBool As Boolean
        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, Nothing)

        Debug.Print("")
        Debug.Print(MyREST.LogData)
        Debug.Print("")

        Dim ResponseMessage As String = ""

        If RetBool = True Then
            Dim MyXmlDocument As System.Xml.XmlDocument
            Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim MyXmlNode As System.Xml.XmlNode
            Dim MyXmlNodeList As System.Xml.XmlNodeList

            MyXmlDocument = New System.Xml.XmlDocument
            MyXmlDocument.LoadXml(MyREST.ResponseString)

            MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
            MyXmlNamespaceManager.AddNamespace("amz", "http://elasticloadbalancing.amazonaws.com/doc/2010-07-01/")

            MyXmlNodeList = MyXmlDocument.SelectNodes("amz:DescribeLoadBalancersResponse/amz:DescribeLoadBalancersResult/amz:LoadBalancerDescriptions/amz:member", MyXmlNamespaceManager)

            If MyXmlNodeList.Count = 0 Then
                ResponseMessage = "No load balancers exist."
            Else

                For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:LoadBalancerName", MyXmlNamespaceManager)
                    ResponseMessage += "LoadBalancerName = " & MyXmlNode.InnerText

                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:CreatedTime", MyXmlNamespaceManager)
                    ResponseMessage += "   CreatedTime = " & MyXmlNode.InnerText

                    ResponseMessage += vbCrLf
                Next
            End If

            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)

        Else
            ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub

    Private Sub ButtonFPSGetAccountActivity_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonFPSGetAccountActivity.Click
        sender.Enabled = False

        'Amazon Flexible Payments Service (FPS and ASP), GetAccountActivity: http://docs.amazonwebservices.com/AmazonFPS/latest/FPSAccountManagementGuide/GetAccountActivity.html

        Dim MyREST As New SprightlySoftAWS.REST

        'get all transactions in the past day
        Dim StartDate As Date
        StartDate = Now.AddDays(-1)

        Dim RequestURL As String
        RequestURL = "https://fps.amazonaws.com/?Action=GetAccountActivity&Version=2008-09-17"
        RequestURL += "&AWSAccessKeyId=" & System.Uri.EscapeDataString(TextBoxAWSAccessKeyId.Text) & "&SignatureVersion=2&SignatureMethod=HmacSHA256&Timestamp=" & Uri.EscapeDataString(DateTime.UtcNow.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))
        RequestURL += "&StartDate=" & Uri.EscapeDataString(StartDate.ToUniversalTime.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))

        Dim RequestMethod As String
        RequestMethod = "GET"

        Dim SignatureValue As String
        SignatureValue = MyREST.GetSignatureVersion2Value(RequestURL, RequestMethod, "", TextBoxAWSSecretAccessKey.Text)

        RequestURL += "&Signature=" & System.Uri.EscapeDataString(SignatureValue)

        Dim RetBool As Boolean
        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, Nothing)

        Debug.Print("")
        Debug.Print(MyREST.LogData)
        Debug.Print("")

        Dim ResponseMessage As String = ""

        If RetBool = True Then
            Dim MyXmlDocument As System.Xml.XmlDocument
            Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim MyXmlNode As System.Xml.XmlNode
            Dim MyXmlNodeList As System.Xml.XmlNodeList

            MyXmlDocument = New System.Xml.XmlDocument
            MyXmlDocument.LoadXml(MyREST.ResponseString)

            MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
            MyXmlNamespaceManager.AddNamespace("amz", "http://fps.amazonaws.com/doc/2008-09-17/")

            MyXmlNodeList = MyXmlDocument.SelectNodes("amz:GetAccountActivityResponse/amz:GetAccountActivityResult/amz:Transaction", MyXmlNamespaceManager)

            If MyXmlNodeList.Count = 0 Then
                ResponseMessage = "No transactions exist."
            Else

                For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:TransactionId", MyXmlNamespaceManager)
                    ResponseMessage += "TransactionId = " & MyXmlNode.InnerText

                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:DateReceived", MyXmlNamespaceManager)
                    ResponseMessage += "   DateReceived = " & MyXmlNode.InnerText

                    ResponseMessage += vbCrLf
                Next
            End If

            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)

        Else
            ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub

    Private Sub ButtonDevPayGetActiveSubscriptionsByPid_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonDevPayGetActiveSubscriptionsByPid.Click
        sender.Enabled = False

        'Amazon DevPay, GetActiveSubscriptionsByPid: http://docs.amazonwebservices.com/AmazonDevPay/latest/DevPayDeveloperGuide/GetActiveSubscriptionsByPid.html

        Dim MyREST As New SprightlySoftAWS.REST

        Dim RequestURL As String
        RequestURL = "https://ls.amazonaws.com/?Action=GetActiveSubscriptionsByPid&Version=2008-04-28"
        RequestURL += "&AWSAccessKeyId=" & System.Uri.EscapeDataString(TextBoxAWSAccessKeyId.Text) & "&SignatureVersion=1&Timestamp=" & Uri.EscapeDataString(DateTime.UtcNow.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))
        RequestURL += "&PersistentIdentifier=PMNGLKRRYHLOXDQKEMKLRTBAULA"

        Dim RequestMethod As String
        RequestMethod = "GET"

        Dim SignatureValue As String
        SignatureValue = MyREST.GetDevPaySignatureValue(RequestURL, TextBoxAWSSecretAccessKey.Text)

        RequestURL += "&Signature=" & System.Uri.EscapeDataString(SignatureValue)

        Dim RetBool As Boolean
        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, Nothing)

        Debug.Print("")
        Debug.Print(MyREST.LogData)
        Debug.Print("")

        Dim ResponseMessage As String = ""

        If RetBool = True Then
            Dim MyXmlDocument As System.Xml.XmlDocument
            Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim MyXmlNodeList As System.Xml.XmlNodeList

            MyXmlDocument = New System.Xml.XmlDocument
            MyXmlDocument.LoadXml(MyREST.ResponseString)

            MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
            MyXmlNamespaceManager.AddNamespace("amz", "http://ls.amazonaws.com/doc/2008-04-28/")

            MyXmlNodeList = MyXmlDocument.SelectNodes("amz:GetActiveSubscriptionsByPidResponse/amz:GetActiveSubscriptionsByPidResult/amz:ProductCode", MyXmlNamespaceManager)

            If MyXmlNodeList.Count = 0 Then
                ResponseMessage = "No products exist."
            Else

                For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                    ResponseMessage += "ProductCode = " & ItemXmlNode.InnerText

                    ResponseMessage += vbCrLf
                Next
            End If

            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)

        Else
            ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub

    Private Sub ButtonS3GETService_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonS3GETService.Click
        sender.Enabled = False

        'Amazon Simple Storage Service (S3), GET Service: http://docs.amazonwebservices.com/AmazonS3/latest/API/RESTServiceGET.html

        'create an instance of the REST class
        Dim MyREST As New SprightlySoftAWS.REST

        Dim RequestURL As String
        'Build the URL to call. Do not specify a bucket name or key name when listing all buckets
        RequestURL = MyREST.BuildS3RequestURL(True, "s3.amazonaws.com", "", "", "")

        Dim RequestMethod As String
        RequestMethod = "GET"

        Dim ExtraRequestHeaders As New Dictionary(Of String, String)
        'add a date header
        ExtraRequestHeaders.Add("x-amz-date", DateTime.UtcNow.ToString("r"))

        Dim AuthorizationValue As String
        'generate the authorization header value
        AuthorizationValue = MyREST.GetS3AuthorizationValue(RequestURL, RequestMethod, ExtraRequestHeaders, TextBoxAWSAccessKeyId.Text, TextBoxAWSSecretAccessKey.Text)
        'add the authorization header
        ExtraRequestHeaders.Add("Authorization", AuthorizationValue)

        'call MakeRequest to submit the REST request
        Dim RetBool As Boolean
        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, ExtraRequestHeaders, "")

        'print the log data to the Output window
        Debug.Print("")
        Debug.Print(MyREST.LogData)
        Debug.Print("")

        Dim ResponseMessage As String = ""

        'check if the result of the operation was successful
        If RetBool = True Then

            Dim MyXmlDocument As System.Xml.XmlDocument
            Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim MyXmlNode As System.Xml.XmlNode
            Dim MyXmlNodeList As System.Xml.XmlNodeList

            'load the response XML from Amazon into an XmlDocument
            MyXmlDocument = New System.Xml.XmlDocument
            MyXmlDocument.LoadXml(MyREST.ResponseString)

            'specify the namespace for XML path expressions
            MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
            MyXmlNamespaceManager.AddNamespace("amz", "http://s3.amazonaws.com/doc/2006-03-01/")

            'extract all values from the response XML
            MyXmlNode = MyXmlDocument.SelectSingleNode("amz:ListAllMyBucketsResult/amz:Owner/amz:ID", MyXmlNamespaceManager)
            ResponseMessage = ResponseMessage & "Owner ID = " & MyXmlNode.InnerText & vbCrLf

            MyXmlNode = MyXmlDocument.SelectSingleNode("amz:ListAllMyBucketsResult/amz:Owner/amz:DisplayName", MyXmlNamespaceManager)
            ResponseMessage = ResponseMessage & "Owner DisplayName = " & MyXmlNode.InnerText & vbCrLf
            ResponseMessage = ResponseMessage & vbCrLf

            MyXmlNodeList = MyXmlDocument.SelectNodes("amz:ListAllMyBucketsResult/amz:Buckets/amz:Bucket", MyXmlNamespaceManager)

            For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                MyXmlNode = ItemXmlNode.SelectSingleNode("amz:Name", MyXmlNamespaceManager)
                ResponseMessage = ResponseMessage & "Bucket Name = " & MyXmlNode.InnerText

                MyXmlNode = ItemXmlNode.SelectSingleNode("amz:CreationDate", MyXmlNamespaceManager)
                ResponseMessage = ResponseMessage & "     CreationDate = " & CDate(MyXmlNode.InnerText) & vbCrLf
            Next

            'display values from the XML in the DialogOutput form
            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)

        Else
            'build an error message
            ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

            'display the error message
            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub

    Private Sub ButtonAWSImportExportListJobs_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonAWSImportExportListJobs.Click
        sender.Enabled = False

        'AWS Import/Export, ListJobs: http://docs.amazonwebservices.com/AWSImportExport/latest/API/WebListJobs.html

        Dim MyREST As New SprightlySoftAWS.REST

        Dim RequestURL As String
        RequestURL = "https://importexport.amazonaws.com"

        Dim PostData As String
        PostData = "Operation=ListJobs&Version=2010-06-01"
        PostData += "&AWSAccessKeyId=" & System.Uri.EscapeDataString(TextBoxAWSAccessKeyId.Text) & "&SignatureVersion=2&SignatureMethod=HmacSHA256&Timestamp=" & Uri.EscapeDataString(DateTime.UtcNow.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))

        Dim RequestMethod As String
        RequestMethod = "POST"

        Dim SignatureValue As String
        SignatureValue = MyREST.GetSignatureVersion2Value(RequestURL, RequestMethod, PostData, TextBoxAWSSecretAccessKey.Text)

        PostData += "&Signature=" & System.Uri.EscapeDataString(SignatureValue)

        Dim ExtraRequestHeaders As New Dictionary(Of String, String)
        ExtraRequestHeaders.Add("Content-Type", "application/x-www-form-urlencoded; charset=utf-8")

        Dim RetBool As Boolean
        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, ExtraRequestHeaders, PostData)

        Debug.Print("")
        Debug.Print(MyREST.LogData)
        Debug.Print("")

        Dim ResponseMessage As String = ""

        If RetBool = True Then
            Dim MyXmlDocument As System.Xml.XmlDocument
            Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim MyXmlNode As System.Xml.XmlNode
            Dim MyXmlNodeList As System.Xml.XmlNodeList

            MyXmlDocument = New System.Xml.XmlDocument
            MyXmlDocument.LoadXml(MyREST.ResponseString)

            MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
            MyXmlNamespaceManager.AddNamespace("amz", "http://importexport.amazonaws.com/doc/2010-06-01/")

            MyXmlNodeList = MyXmlDocument.SelectNodes("amz:ListJobsResponse/amz:ListJobsResult/amz:Jobs/amz:member", MyXmlNamespaceManager)

            If MyXmlNodeList.Count = 0 Then
                ResponseMessage = "No jobs exist."
            Else

                For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:JobId", MyXmlNamespaceManager)
                    ResponseMessage += "JobId = " & MyXmlNode.InnerText

                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:CreationDate", MyXmlNamespaceManager)
                    ResponseMessage += "   CreationDate = " & MyXmlNode.InnerText

                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:IsCanceled", MyXmlNamespaceManager)
                    ResponseMessage += "   IsCanceled = " & MyXmlNode.InnerText

                    ResponseMessage += vbCrLf
                Next
            End If

            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)

        Else
            ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub

    Private Sub ButtonAlexaWebInformationServiceUrlInfo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonAlexaWebInformationServiceUrlInfo.Click
        sender.Enabled = False

        'Alexa Web Information Service, UrlInfo: http://docs.amazonwebservices.com/AlexaWebInfoService/2005-07-11/ApiReference_UrlInfoAction.html

        Dim MyREST As New SprightlySoftAWS.REST

        Dim RequestURL As String
        RequestURL = "http://awis.amazonaws.com/?Action=UrlInfo&Version=2005-07-11"
        RequestURL += "&AWSAccessKeyId=" & System.Uri.EscapeDataString(TextBoxAWSAccessKeyId.Text) & "&Timestamp=" & Uri.EscapeDataString(DateTime.UtcNow.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))
        RequestURL += "&ResponseGroup=Rank&Url=yahoo.com"

        Dim RequestMethod As String
        RequestMethod = "GET"

        Dim SignatureValue As String
        SignatureValue = MyREST.GetAlexaSignatureValue(RequestURL, TextBoxAWSSecretAccessKey.Text)

        RequestURL += "&Signature=" & System.Uri.EscapeDataString(SignatureValue)

        Dim RetBool As Boolean
        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, Nothing)

        Debug.Print("")
        Debug.Print(MyREST.LogData)
        Debug.Print("")

        Dim ResponseMessage As String = ""

        If RetBool = True Then
            Dim EnvelopeXmlDocument As System.Xml.XmlDocument
            Dim EnvelopeXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim EnvelopeXmlNode As System.Xml.XmlNode
            Dim MyXmlDocument As System.Xml.XmlDocument
            Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim MyXmlNode As System.Xml.XmlNode

            EnvelopeXmlDocument = New System.Xml.XmlDocument
            EnvelopeXmlDocument.LoadXml(MyREST.ResponseString)

            EnvelopeXmlNamespaceManager = New System.Xml.XmlNamespaceManager(EnvelopeXmlDocument.NameTable)
            EnvelopeXmlNamespaceManager.AddNamespace("env", "http://alexa.amazonaws.com/doc/2005-10-05/")

            EnvelopeXmlNode = EnvelopeXmlDocument.SelectSingleNode("env:UrlInfoResponse", EnvelopeXmlNamespaceManager)

            MyXmlDocument = New System.Xml.XmlDocument
            MyXmlDocument.LoadXml(EnvelopeXmlNode.InnerXml)

            MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
            MyXmlNamespaceManager.AddNamespace("amz", "http://awis.amazonaws.com/doc/2005-07-11")

            MyXmlNode = MyXmlDocument.SelectSingleNode("amz:Response/amz:UrlInfoResult/amz:Alexa/amz:ContentData/amz:SiteData/amz:Title", MyXmlNamespaceManager)

            If IsNothing(MyXmlNode) = True Then
                ResponseMessage = "Url title does not exist."
            Else
                ResponseMessage = "Title = " & MyXmlNode.InnerText
            End If

            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)

        Else
            ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub

    Private Sub ButtonAlexaTopSitesTopSites_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonAlexaTopSitesTopSites.Click
        sender.Enabled = False

        'Alexa Top Sites , TopSites: http://docs.amazonwebservices.com/AlexaTopSites/2005-11-21_2/ApiReference/TopSitesAction.html#ResponseGroups

        Dim MyREST As New SprightlySoftAWS.REST

        Dim RequestURL As String
        RequestURL = "http://ats.amazonaws.com/?Action=TopSites"
        RequestURL += "&AWSAccessKeyId=" & System.Uri.EscapeDataString(TextBoxAWSAccessKeyId.Text) & "&Timestamp=" & Uri.EscapeDataString(DateTime.UtcNow.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))
        RequestURL += "&ResponseGroup=Country"

        Dim RequestMethod As String
        RequestMethod = "GET"

        Dim SignatureValue As String
        SignatureValue = MyREST.GetAlexaSignatureValue(RequestURL, TextBoxAWSSecretAccessKey.Text)

        RequestURL += "&Signature=" & System.Uri.EscapeDataString(SignatureValue)

        Dim RetBool As Boolean
        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, Nothing)

        Debug.Print("")
        Debug.Print(MyREST.LogData)
        Debug.Print("")

        Dim ResponseMessage As String = ""

        If RetBool = True Then
            Dim EnvelopeXmlDocument As System.Xml.XmlDocument
            Dim EnvelopeXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim EnvelopeXmlNode As System.Xml.XmlNode
            Dim MyXmlDocument As System.Xml.XmlDocument
            Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim MyXmlNode As System.Xml.XmlNode
            Dim MyXmlNodeList As System.Xml.XmlNodeList

            EnvelopeXmlDocument = New System.Xml.XmlDocument
            EnvelopeXmlDocument.LoadXml(MyREST.ResponseString)

            EnvelopeXmlNamespaceManager = New System.Xml.XmlNamespaceManager(EnvelopeXmlDocument.NameTable)
            EnvelopeXmlNamespaceManager.AddNamespace("env", "http://ats.amazonaws.com/doc/2005-10-05/")

            EnvelopeXmlNode = EnvelopeXmlDocument.SelectSingleNode("env:TopSitesResponse", EnvelopeXmlNamespaceManager)

            MyXmlDocument = New System.Xml.XmlDocument
            MyXmlDocument.LoadXml(EnvelopeXmlNode.InnerXml)

            MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
            MyXmlNamespaceManager.AddNamespace("amz", "http://ats.amazonaws.com/doc/2005-11-21")

            MyXmlNodeList = MyXmlDocument.SelectNodes("amz:Response/amz:TopSitesResult/amz:Alexa/amz:TopSites/amz:Country", MyXmlNamespaceManager)

            If MyXmlNodeList.Count = 0 Then
                ResponseMessage = "No countries exist."
            Else

                For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:CountryName", MyXmlNamespaceManager)
                    ResponseMessage += "CountryName = " & MyXmlNode.InnerText

                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:CountryCode", MyXmlNamespaceManager)
                    ResponseMessage += "   CountryCode = " & MyXmlNode.InnerText

                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:TotalSites", MyXmlNamespaceManager)
                    ResponseMessage += "   TotalSites = " & MyXmlNode.InnerText

                    ResponseMessage += vbCrLf
                Next
            End If

            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)

        Else
            ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub

    Private Sub ButtonMechanicalTurkGetAccountBalance_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonMechanicalTurkGetAccountBalance.Click
        sender.Enabled = False

        'Amazon Mechanical Turk, GetAccountBalance: http://docs.amazonwebservices.com/AWSMechTurk/latest/AWSMturkAPI/ApiReference_GetAccountBalanceOperation.html

        Dim MyREST As New SprightlySoftAWS.REST

        Dim RequestURL As String
        RequestURL = "https://mechanicalturk.amazonaws.com/?Service=AWSMechanicalTurkRequester&Operation=GetAccountBalance&Version=2008-08-02"
        RequestURL += "&AWSAccessKeyId=" & System.Uri.EscapeDataString(TextBoxAWSAccessKeyId.Text) & "&Timestamp=" & Uri.EscapeDataString(DateTime.UtcNow.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))

        Dim RequestMethod As String
        RequestMethod = "GET"

        Dim SignatureValue As String
        SignatureValue = MyREST.GetMechanicalTurkSignatureValue(RequestURL, TextBoxAWSSecretAccessKey.Text)

        RequestURL += "&Signature=" & System.Uri.EscapeDataString(SignatureValue)

        Dim RetBool As Boolean
        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, Nothing)

        Debug.Print("")
        Debug.Print(MyREST.LogData)
        Debug.Print("")

        Dim ResponseMessage As String = ""

        If RetBool = True Then
            Dim MyXmlDocument As System.Xml.XmlDocument
            Dim MyXmlNode As System.Xml.XmlNode

            MyXmlDocument = New System.Xml.XmlDocument
            MyXmlDocument.LoadXml(MyREST.ResponseString)

            MyXmlNode = MyXmlDocument.SelectSingleNode("GetAccountBalanceResponse/GetAccountBalanceResult/AvailableBalance/Amount")

            If IsNothing(MyXmlNode) = True Then
                ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

                DialogOutput.Text = "Error"
                DialogOutput.TextBoxOutput.Text = ResponseMessage
                DialogOutput.ShowDialog(Me)
            Else
                ResponseMessage = "Amount = " & MyXmlNode.InnerText

                DialogOutput.Text = "Success"
                DialogOutput.TextBoxOutput.Text = ResponseMessage
                DialogOutput.ShowDialog(Me)
            End If

        Else
            ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub

    Private Sub ButtonProductAdvertisingAPIItemSearch_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonProductAdvertisingAPIItemSearch.Click
        sender.Enabled = False

        'Product Advertising API, ItemSearch: http://docs.aws.amazon.com/AWSECommerceService/2011-08-01/DG/ItemSearch.html

        Dim AssociateTag As String
        AssociateTag = InputBox("Enter your AssociateTag. To find your AssociateTag log into https://affiliate-program.amazon.com/ and look for Tracking ID.", "", My.Settings.AssociateTag)

        If AssociateTag <> "" Then
            My.Settings.AssociateTag = AssociateTag

            Dim MyREST As New SprightlySoftAWS.REST

            Dim RequestURL As String
            RequestURL = "https://webservices.amazon.com/onca/xml?Service=AWSECommerceService&Operation=ItemSearch&Version=2011-08-01"
            RequestURL += "&AWSAccessKeyId=" & System.Uri.EscapeDataString(TextBoxAWSAccessKeyId.Text) & "&SignatureVersion=2&SignatureMethod=HmacSHA256&Timestamp=" & Uri.EscapeDataString(DateTime.UtcNow.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))
            RequestURL += "&AssociateTag=" & System.Uri.EscapeDataString(AssociateTag)
            RequestURL += "&Actor=" & System.Uri.EscapeDataString("Tom Cruise")
            RequestURL += "&SearchIndex=DVD"
            RequestURL += "&ResponseGroup=" & System.Uri.EscapeDataString("ItemAttributes,Reviews")
            RequestURL += "&Sort=salesrank"
            RequestURL += "&ItemPage=1"

            Dim RequestMethod As String
            RequestMethod = "GET"

            Dim SignatureValue As String
            SignatureValue = MyREST.GetSignatureVersion2Value(RequestURL, RequestMethod, "", TextBoxAWSSecretAccessKey.Text)

            RequestURL += "&Signature=" & System.Uri.EscapeDataString(SignatureValue)

            Dim RetBool As Boolean
            RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, Nothing)

            Debug.Print("")
            Debug.Print(MyREST.LogData)
            Debug.Print("")

            Dim ResponseMessage As String = ""

            If RetBool = True Then
                Dim MyXmlDocument As System.Xml.XmlDocument
                Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
                Dim MyXmlNode As System.Xml.XmlNode
                Dim MyXmlNodeList As System.Xml.XmlNodeList

                MyXmlDocument = New System.Xml.XmlDocument
                MyXmlDocument.LoadXml(MyREST.ResponseString)

                MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
                MyXmlNamespaceManager.AddNamespace("amz", "http://webservices.amazon.com/AWSECommerceService/2011-08-01")

                MyXmlNodeList = MyXmlDocument.SelectNodes("amz:ItemSearchResponse/amz:Items/amz:Item", MyXmlNamespaceManager)

                If MyXmlNodeList.Count = 0 Then
                    ResponseMessage = "No items exist."
                Else

                    For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                        MyXmlNode = ItemXmlNode.SelectSingleNode("amz:ItemAttributes/amz:Title", MyXmlNamespaceManager)
                        ResponseMessage += "Title = " & MyXmlNode.InnerText

                        MyXmlNode = ItemXmlNode.SelectSingleNode("amz:ItemAttributes/amz:ListPrice/amz:FormattedPrice", MyXmlNamespaceManager)
                        If IsNothing(MyXmlNode) = False Then
                            ResponseMessage += "   ListPrice = " & MyXmlNode.InnerText
                        End If

                        ResponseMessage += vbCrLf
                    Next
                End If

                DialogOutput.Text = "Success"
                DialogOutput.TextBoxOutput.Text = ResponseMessage
                DialogOutput.ShowDialog(Me)

            Else
                ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

                DialogOutput.Text = "Error"
                DialogOutput.TextBoxOutput.Text = ResponseMessage
                DialogOutput.ShowDialog(Me)
            End If
        End If

        sender.Enabled = True
    End Sub

    Private Sub ButtonProductAdvertisingAPIItemLookup_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonProductAdvertisingAPIItemLookup.Click
        sender.Enabled = False

        'Product Advertising API, ItemLookup: http://docs.aws.amazon.com/AWSECommerceService/2011-08-01/DG/ItemLookup.html

        Dim AssociateTag As String
        AssociateTag = InputBox("Enter your AssociateTag. To find your AssociateTag log into https://affiliate-program.amazon.com/ and look for Tracking ID.", "", My.Settings.AssociateTag)

        If AssociateTag <> "" Then
            My.Settings.AssociateTag = AssociateTag

            Dim MyREST As New SprightlySoftAWS.REST

            Dim RequestURL As String
            RequestURL = "https://webservices.amazon.com/onca/xml?Service=AWSECommerceService&Operation=ItemLookup&Version=2011-08-01"
            RequestURL += "&AWSAccessKeyId=" & System.Uri.EscapeDataString(TextBoxAWSAccessKeyId.Text) & "&SignatureVersion=2&SignatureMethod=HmacSHA256&Timestamp=" & Uri.EscapeDataString(DateTime.UtcNow.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))
            RequestURL += "&AssociateTag=" & System.Uri.EscapeDataString(AssociateTag)
            RequestURL += "&ItemId=9781904233657"
            RequestURL += "&IdType=ISBN"
            RequestURL += "&SearchIndex=Books"

            Dim RequestMethod As String
            RequestMethod = "GET"

            Dim SignatureValue As String
            SignatureValue = MyREST.GetSignatureVersion2Value(RequestURL, RequestMethod, "", TextBoxAWSSecretAccessKey.Text)

            RequestURL += "&Signature=" & System.Uri.EscapeDataString(SignatureValue)

            Dim RetBool As Boolean
            RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, Nothing)

            Debug.Print("")
            Debug.Print(MyREST.LogData)
            Debug.Print("")

            Dim ResponseMessage As String = ""

            If RetBool = True Then
                Dim MyXmlDocument As System.Xml.XmlDocument
                Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
                Dim MyXmlNode As System.Xml.XmlNode
                Dim MyXmlNodeList As System.Xml.XmlNodeList

                MyXmlDocument = New System.Xml.XmlDocument
                MyXmlDocument.LoadXml(MyREST.ResponseString)

                MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
                MyXmlNamespaceManager.AddNamespace("amz", "http://webservices.amazon.com/AWSECommerceService/2011-08-01")

                MyXmlNodeList = MyXmlDocument.SelectNodes("amz:ItemLookupResponse/amz:Items/amz:Item", MyXmlNamespaceManager)

                If MyXmlNodeList.Count = 0 Then
                    ResponseMessage = "Item not found."
                Else

                    For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                        MyXmlNode = ItemXmlNode.SelectSingleNode("amz:ItemAttributes/amz:Title", MyXmlNamespaceManager)
                        ResponseMessage += "Title = " & MyXmlNode.InnerText

                        ResponseMessage += vbCrLf
                    Next
                End If

                DialogOutput.Text = "Success"
                DialogOutput.TextBoxOutput.Text = ResponseMessage
                DialogOutput.ShowDialog(Me)

            Else
                ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

                DialogOutput.Text = "Error"
                DialogOutput.TextBoxOutput.Text = ResponseMessage
                DialogOutput.ShowDialog(Me)
            End If
        End If

        sender.Enabled = True
    End Sub


    Private Sub ButtonAmazonRoute53ListHostedZones_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonAmazonRoute53ListHostedZones.Click
        sender.Enabled = False

        'Amazon Route 53, ListHostedZones: http://docs.amazonwebservices.com/Route53/latest/APIReference/API_ListHostedZones.html

        Dim MyREST As New SprightlySoftAWS.REST

        Dim RequestURL As String
        RequestURL = "https://route53.amazonaws.com/2010-10-01/hostedzone"

        Dim RequestMethod As String
        RequestMethod = "GET"

        'Create the date that will be used for the x-amz-date hreader and X-Amzn-Authorization signature value.
        Dim MyDate As String
        MyDate = DateTime.UtcNow.ToString("r")

        'Add the date header.
        Dim ExtraRequestHeaders As Dictionary(Of String, String) = New Dictionary(Of String, String)
        ExtraRequestHeaders.Add("x-amz-date", MyDate)

        Dim AuthorizationValue As String
        AuthorizationValue = MyREST.GetAWS3AuthorizationValue(TextBoxAWSAccessKeyId.Text, TextBoxAWSSecretAccessKey.Text, MyDate)

        'Add the authorization header.
        ExtraRequestHeaders.Add("X-Amzn-Authorization", AuthorizationValue)

        Dim RetBool As Boolean
        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, ExtraRequestHeaders)

        Debug.Print("")
        Debug.Print(MyREST.LogData)
        Debug.Print("")

        Dim ResponseMessage As String = ""

        If RetBool = True Then
            Dim MyXmlDocument As System.Xml.XmlDocument
            Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim MyXmlNode As System.Xml.XmlNode
            Dim MyXmlNodeList As System.Xml.XmlNodeList

            MyXmlDocument = New System.Xml.XmlDocument
            MyXmlDocument.LoadXml(MyREST.ResponseString)

            MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
            MyXmlNamespaceManager.AddNamespace("amz", "https://route53.amazonaws.com/doc/2010-10-01/")

            MyXmlNodeList = MyXmlDocument.SelectNodes("amz:ListHostedZonesResponse/amz:HostedZones/amz:HostedZone", MyXmlNamespaceManager)

            If MyXmlNodeList.Count = 0 Then
                ResponseMessage = "No hosted zones exist."
            Else
                For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:Id", MyXmlNamespaceManager)
                    ResponseMessage += "Id = " & MyXmlNode.InnerText & vbCrLf

                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:Name", MyXmlNamespaceManager)
                    ResponseMessage += "Name = " & MyXmlNode.InnerText & vbCrLf

                    ResponseMessage += vbCrLf
                Next
            End If

            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)

        Else
            ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub

    Private Sub ButtonAWSElasticBeanstalkDescribeApplications_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonAWSElasticBeanstalkDescribeApplications.Click
        sender.Enabled = False

        'AWS Elastic Beanstalk, DescribeApplications: http://docs.amazonwebservices.com/elasticbeanstalk/latest/api/API_DescribeApplications.html

        Dim MyREST As New SprightlySoftAWS.REST

        Dim RequestURL As String
        RequestURL = "https://elasticbeanstalk.us-east-1.amazonaws.com/?Operation=DescribeApplications"
        RequestURL += "&AWSAccessKeyId=" & System.Uri.EscapeDataString(TextBoxAWSAccessKeyId.Text) & "&SignatureVersion=2&SignatureMethod=HmacSHA256&Timestamp=" & Uri.EscapeDataString(DateTime.UtcNow.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))

        Dim RequestMethod As String
        RequestMethod = "GET"

        Dim SignatureValue As String
        SignatureValue = MyREST.GetSignatureVersion2Value(RequestURL, RequestMethod, "", TextBoxAWSSecretAccessKey.Text)

        RequestURL += "&Signature=" & System.Uri.EscapeDataString(SignatureValue)

        Dim RetBool As Boolean
        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, Nothing)

        Debug.Print("")
        Debug.Print(MyREST.LogData)
        Debug.Print("")

        Dim ResponseMessage As String = ""

        If RetBool = True Then
            Dim MyXmlDocument As System.Xml.XmlDocument
            Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim MyXmlNode As System.Xml.XmlNode
            Dim MyXmlNodeList As System.Xml.XmlNodeList

            MyXmlDocument = New System.Xml.XmlDocument
            MyXmlDocument.LoadXml(MyREST.ResponseString)

            MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
            MyXmlNamespaceManager.AddNamespace("amz", "http://elasticbeanstalk.amazonaws.com/docs/2010-12-01/")

            MyXmlNodeList = MyXmlDocument.SelectNodes("amz:DescribeApplicationsResponse/amz:DescribeApplicationsResult/amz:Applications/amz:member", MyXmlNamespaceManager)

            If MyXmlNodeList.Count = 0 Then
                ResponseMessage = "No applications found."
            Else
                For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:ApplicationName", MyXmlNamespaceManager)
                    ResponseMessage += "ApplicationName = " & MyXmlNode.InnerText

                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:Description", MyXmlNamespaceManager)
                    ResponseMessage += "   Description = " & MyXmlNode.InnerText

                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:DateCreated", MyXmlNamespaceManager)
                    ResponseMessage += "   DateCreated = " & CDate(MyXmlNode.InnerText)

                    ResponseMessage += vbCrLf
                Next
            End If

            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)

        Else
            ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub


    Private Sub ButtonAmazonSimpleEmailServiceListVerifiedEmailAddresses_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonAmazonSimpleEmailServiceListVerifiedEmailAddresses.Click
        sender.Enabled = False

        'Amazon Simple Email Service, ListVerifiedEmailAddresses: http://docs.amazonwebservices.com/ses/latest/APIReference/API_ListVerifiedEmailAddresses.html

        Dim MyREST As New SprightlySoftAWS.REST

        Dim RequestURL As String
        RequestURL = "https://email.us-east-1.amazonaws.com?Action=ListVerifiedEmailAddresses"

        Dim RequestMethod As String
        RequestMethod = "GET"

        'Create the date that will be used for the x-amz-date hreader and X-Amzn-Authorization signature value.
        Dim MyDate As String
        MyDate = DateTime.UtcNow.ToString("r")

        'Add the date header.
        Dim ExtraRequestHeaders As Dictionary(Of String, String) = New Dictionary(Of String, String)
        ExtraRequestHeaders.Add("x-amz-date", MyDate)

        Dim AuthorizationValue As String
        AuthorizationValue = MyREST.GetAWS3AuthorizationValue(TextBoxAWSAccessKeyId.Text, TextBoxAWSSecretAccessKey.Text, MyDate)

        'Add the authorization header.
        ExtraRequestHeaders.Add("X-Amzn-Authorization", AuthorizationValue)

        Dim RetBool As Boolean
        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, ExtraRequestHeaders)

        Debug.Print("")
        Debug.Print(MyREST.LogData)
        Debug.Print("")

        Dim ResponseMessage As String = ""

        If RetBool = True Then
            Dim MyXmlDocument As System.Xml.XmlDocument
            Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim MyXmlNodeList As System.Xml.XmlNodeList

            MyXmlDocument = New System.Xml.XmlDocument
            MyXmlDocument.LoadXml(MyREST.ResponseString)

            MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
            MyXmlNamespaceManager.AddNamespace("amz", "http://ses.amazonaws.com/doc/2010-12-01/")

            MyXmlNodeList = MyXmlDocument.SelectNodes("amz:ListVerifiedEmailAddressesResponse/amz:ListVerifiedEmailAddressesResult/amz:VerifiedEmailAddresses/amz:member", MyXmlNamespaceManager)

            If MyXmlNodeList.Count = 0 Then
                ResponseMessage = "No verified email addresses exist."
            Else
                For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                    ResponseMessage += "EmailAddress = " & ItemXmlNode.InnerText & vbCrLf
                Next
            End If

            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)

        Else
            ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub

    Private Sub ButtonAWSCloudFormationDescribeStacks_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonAWSCloudFormationDescribeStacks.Click
        sender.Enabled = False

        'AWS CloudFormation, DescribeStacks: http://docs.amazonwebservices.com/AWSCloudFormation/latest/APIReference/API_DescribeStacks.html

        Dim MyREST As New SprightlySoftAWS.REST

        Dim RequestURL As String
        RequestURL = "https://cloudformation.us-east-1.amazonaws.com/?Action=DescribeStacks&Version=2010-05-15"
        RequestURL += "&AWSAccessKeyId=" & System.Uri.EscapeDataString(TextBoxAWSAccessKeyId.Text) & "&SignatureVersion=2&SignatureMethod=HmacSHA256&Timestamp=" & Uri.EscapeDataString(DateTime.UtcNow.ToString("yyyy-MM-dd\THH:mm:ss.fff\Z"))

        Dim RequestMethod As String
        RequestMethod = "GET"

        Dim SignatureValue As String
        SignatureValue = MyREST.GetSignatureVersion2Value(RequestURL, RequestMethod, "", TextBoxAWSSecretAccessKey.Text)

        RequestURL += "&Signature=" & System.Uri.EscapeDataString(SignatureValue)

        Dim RetBool As Boolean
        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, Nothing)

        Debug.Print("")
        Debug.Print(MyREST.LogData)
        Debug.Print("")

        Dim ResponseMessage As String = ""

        If RetBool = True Then
            Dim MyXmlDocument As System.Xml.XmlDocument
            Dim MyXmlNamespaceManager As System.Xml.XmlNamespaceManager
            Dim MyXmlNode As System.Xml.XmlNode
            Dim MyXmlNodeList As System.Xml.XmlNodeList

            MyXmlDocument = New System.Xml.XmlDocument
            MyXmlDocument.LoadXml(MyREST.ResponseString)

            MyXmlNamespaceManager = New System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable)
            MyXmlNamespaceManager.AddNamespace("amz", "http://cloudformation.amazonaws.com/doc/2010-05-15/")

            MyXmlNodeList = MyXmlDocument.SelectNodes("amz:DescribeStacksResult/amz:Stacks", MyXmlNamespaceManager)

            If MyXmlNodeList.Count = 0 Then
                ResponseMessage = "No stacks found."
            Else
                For Each ItemXmlNode As System.Xml.XmlNode In MyXmlNodeList
                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:StackName", MyXmlNamespaceManager)
                    ResponseMessage += "StackName = " & MyXmlNode.InnerText

                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:StackId", MyXmlNamespaceManager)
                    ResponseMessage += "   StackId = " & MyXmlNode.InnerText

                    MyXmlNode = ItemXmlNode.SelectSingleNode("amz:StackStatus", MyXmlNamespaceManager)
                    ResponseMessage += "   StackStatus = " & CDate(MyXmlNode.InnerText)

                    ResponseMessage += vbCrLf
                Next
            End If

            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)

        Else
            ResponseMessage = FormatLogData(MyREST.RequestURL, MyREST.RequestMethod, MyREST.RequestHeaders, MyREST.ResponseStatusCode, MyREST.ResponseStatusDescription, MyREST.ResponseHeaders, MyREST.ResponseStringFormatted, MyREST.ErrorNumber, MyREST.ErrorDescription)

            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub


    Private Sub ButtonGlacierListVaults_Click(sender As System.Object, e As System.EventArgs) Handles ButtonGlacierListVaults.Click
        sender.Enabled = False

        'List Vaults (GET vaults): http://docs.aws.amazon.com/amazonglacier/latest/dev/api-vaults-get.html
        'Regions and Endpoints:  http://docs.aws.amazon.com/general/latest/gr/rande.html#glacier_region

        Dim MyJavaScriptSerializer As System.Web.Script.Serialization.JavaScriptSerializer = New System.Web.Script.Serialization.JavaScriptSerializer()
        MyJavaScriptSerializer.MaxJsonLength = Int32.MaxValue

        Dim QueryString As String
        Dim Marker As String
        Dim UserRequestHeaders As Dictionary(Of String, String)
        Dim RequestURL As String
        Dim RequestMethod As String
        Dim PostData As String
        Dim PostDataHash As String
        Dim MyCalculateHash As SprightlySoftAWS.S3.CalculateHash = New SprightlySoftAWS.S3.CalculateHash
        Dim AuthorizationValue As String
        Dim RetBool As Boolean
        Dim MyREST As SprightlySoftAWS.REST = New SprightlySoftAWS.REST
        Dim ErrorNumber As Integer
        Dim ErrorDescription As String
        Dim LogData As String
        Dim FullRequestHeaders As Dictionary(Of String, String)
        Dim ResponseStatusCode As Integer
        Dim ResponseStatusDescription As String
        Dim FullResponseHeaders As Dictionary(Of String, String)
        Dim ResponseString As String
        Dim MyObjectDictionary As Dictionary(Of String, Object)
        Dim ResponseMessage As String = ""
        Dim VaultListObjectArray As Object()

        'list all vaults in all regions
        Dim RegionsArrayList As ArrayList = New ArrayList
        RegionsArrayList.Add("us-east-1")
        RegionsArrayList.Add("us-west-2")
        RegionsArrayList.Add("us-west-1")
        RegionsArrayList.Add("eu-west-1")
        RegionsArrayList.Add("ap-northeast-1")

        For Each MyRegion As String In RegionsArrayList
            Marker = ""

            Do
                QueryString = ""
                If Marker <> "" Then
                    QueryString += "?marker=" + Marker
                End If

                UserRequestHeaders = New Dictionary(Of String, String)
                UserRequestHeaders.Add("x-amz-date", DateTime.UtcNow.ToString("yyyyMMddTHHmmssZ", System.Globalization.CultureInfo.InvariantCulture))
                UserRequestHeaders.Add("x-amz-glacier-version", "2012-06-01")

                RequestURL = "https://glacier." + MyRegion + ".amazonaws.com/-/vaults" + QueryString
                RequestMethod = "GET"
                PostData = ""
                PostDataHash = MyCalculateHash.CalculateSHA256FromString(PostData)

                AuthorizationValue = MyREST.GetAWSSignatureVersion4Value(UserRequestHeaders, RequestURL, "glacier", MyRegion, RequestMethod, PostDataHash, TextBoxAWSAccessKeyId.Text, TextBoxAWSSecretAccessKey.Text)
                UserRequestHeaders.Add("Authorization", AuthorizationValue)

                RetBool = MakeAWSRequest(RequestURL, RequestMethod, UserRequestHeaders, PostData, ErrorNumber, ErrorDescription, LogData, FullRequestHeaders, ResponseStatusCode, ResponseStatusDescription, FullResponseHeaders, ResponseString)
                Debug.Print(LogData)

                If RetBool = True Then
                    MyObjectDictionary = MyJavaScriptSerializer.DeserializeObject(ResponseString)
                    VaultListObjectArray = MyObjectDictionary("VaultList")

                    If VaultListObjectArray.Length > 0 Then
                        For Each VaultListDictionary As Dictionary(Of String, Object) In VaultListObjectArray
                            ResponseMessage += VaultListDictionary("VaultName") + vbCrLf
                        Next
                    End If

                    If IsNothing(MyObjectDictionary("Marker")) = True Then
                        Exit Do
                    Else
                        Marker = MyObjectDictionary("Marker").ToString()
                    End If

                Else
                    Exit For
                End If

            Loop
        Next

        If RetBool = True Then
            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        Else
            ResponseMessage = FormatLogData(RequestURL, RequestMethod, FullRequestHeaders, ResponseStatusCode, ResponseStatusDescription, FullResponseHeaders, ResponseString, ErrorNumber, ErrorDescription)
            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub


    Private Function MakeAWSRequest(ByVal RequestURL As String, ByVal RequestMethod As String, ByVal UserRequestHeaders As Dictionary(Of String, String), ByVal PostData As String, ByRef ErrorNumber As Integer, ByRef ErrorDescription As String, ByRef LogData As String, ByRef FullRequestHeaders As Dictionary(Of String, String), ByRef ResponseStatusCode As Integer, ByRef ResponseStatusDescription As String, ByRef FullResponseHeaders As Dictionary(Of String, String), ByRef ResponseStringFormatted As String) As Boolean

        Dim MyREST As SprightlySoftAWS.REST = New SprightlySoftAWS.REST()
        Dim RetBool As Boolean = True
        LogData = ""

        For i As Integer = 0 To AWSErrorRetries
            LogData += Environment.NewLine
            LogData += "-Request Started " + DateTime.Now.ToString() + Environment.NewLine
            LogData += "-RequestURL=" + RequestURL + Environment.NewLine
            LogData += "-RequestMethod=" + RequestMethod + Environment.NewLine

            If IsNothing(UserRequestHeaders) = False Then
                For Each MyKeyValuePair As KeyValuePair(Of String, String) In UserRequestHeaders
                    LogData += "-RequestHeader=" + MyKeyValuePair.Key + ":" + MyKeyValuePair.Value + Environment.NewLine
                Next
            End If

            LogData += "-RequestPostData=" + PostData + Environment.NewLine

            RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, UserRequestHeaders, PostData)

            LogData += "-Request Complete " + DateTime.Now.ToString() + Environment.NewLine
            LogData += "-ResponseErrorNumber=" + MyREST.ErrorNumber.ToString() + Environment.NewLine
            LogData += "-ResponseErrorDescription=" + MyREST.ErrorDescription + Environment.NewLine
            LogData += "-ResponseStatusCode=" + MyREST.ResponseStatusCode.ToString() + Environment.NewLine
            LogData += "-ResponseStatusDescription=" + MyREST.ResponseStatusDescription + Environment.NewLine

            If IsNothing(MyREST.ResponseHeaders) = False Then
                For Each MyKeyValuePair As KeyValuePair(Of String, String) In MyREST.ResponseHeaders
                    LogData += "-ResponseHeader=" + MyKeyValuePair.Key + ":" + MyKeyValuePair.Value + Environment.NewLine
                Next
            End If

            LogData += "-ResponseString=" + MyREST.ResponseString + Environment.NewLine

            'Set the return values.
            ErrorNumber = MyREST.ErrorNumber
            ErrorDescription = MyREST.ErrorDescription
            FullRequestHeaders = MyREST.RequestHeaders
            ResponseStatusCode = MyREST.ResponseStatusCode
            ResponseStatusDescription = MyREST.ResponseStatusDescription
            FullResponseHeaders = MyREST.ResponseHeaders
            ResponseStringFormatted = MyREST.ResponseStringFormatted

            If RetBool = True Then
                Exit For
            Else
                If MyREST.ResponseStatusCode = 500 OrElse MyREST.ResponseStatusCode = 503 Then
                    'A Service Unavailable response was returned.  Wait and retry.
                    If (i * i < 60) Then
                        System.Threading.Thread.Sleep(1000 * i * i)
                    Else
                        System.Threading.Thread.Sleep(1000 * 60)
                    End If

                ElseIf MyREST.ResponseStatusCode = 0 AndAlso MyREST.ErrorNumber = 1003 Then

                    'Getting the response failed.  This may be a network disconnection.  Wait and retry.
                    If (i * i < 60) Then
                        System.Threading.Thread.Sleep(1000 * i * i)
                    Else
                        System.Threading.Thread.Sleep(1000 * 60)
                    End If
                Else
                    'An error occured but retrying would not solve the problem.
                    Exit For
                End If
            End If

        Next

        Return RetBool

    End Function


    Private Sub ButtonDynamoDBListTables_Click(sender As System.Object, e As System.EventArgs) Handles ButtonDynamoDBListTables.Click
        sender.Enabled = False

        'ListTables:  http://docs.aws.amazon.com/amazondynamodb/latest/APIReference/API_ListTables.html

        Dim MyJavaScriptSerializer As System.Web.Script.Serialization.JavaScriptSerializer = New System.Web.Script.Serialization.JavaScriptSerializer()
        MyJavaScriptSerializer.MaxJsonLength = Int32.MaxValue

        Dim QueryString As String
        Dim Marker As String
        Dim UserRequestHeaders As Dictionary(Of String, String)
        Dim RequestURL As String
        Dim RequestMethod As String
        Dim PostData As String
        Dim PostDataHash As String
        Dim MyCalculateHash As SprightlySoftAWS.S3.CalculateHash = New SprightlySoftAWS.S3.CalculateHash
        Dim AuthorizationValue As String
        Dim RetBool As Boolean
        Dim MyREST As SprightlySoftAWS.REST = New SprightlySoftAWS.REST
        Dim ErrorNumber As Integer
        Dim ErrorDescription As String
        Dim LogData As String
        Dim FullRequestHeaders As Dictionary(Of String, String)
        Dim ResponseStatusCode As Integer
        Dim ResponseStatusDescription As String
        Dim FullResponseHeaders As Dictionary(Of String, String)
        Dim ResponseString As String
        Dim ResponseMessage As String = ""

        Dim MyRegion As String = "us-east-1"

        UserRequestHeaders = New Dictionary(Of String, String)
        UserRequestHeaders.Add("x-amz-date", DateTime.UtcNow.ToString("yyyyMMddTHHmmssZ", System.Globalization.CultureInfo.InvariantCulture))
        UserRequestHeaders.Add("x-amz-target", "DynamoDB_20120810.ListTables")
        UserRequestHeaders.Add("Content-Type", "application/x-amz-json-1.0")

        RequestURL = "https://dynamodb." & MyRegion & ".amazonaws.com"
        RequestMethod = "POST"
        PostData = "{""Limit"": 100}"
        PostDataHash = MyCalculateHash.CalculateSHA256FromString(PostData)
        UserRequestHeaders.Add("x-amz-content-sha256", PostDataHash)

        AuthorizationValue = MyREST.GetAWSSignatureVersion4Value(UserRequestHeaders, RequestURL, "dynamodb", MyRegion, RequestMethod, PostDataHash, TextBoxAWSAccessKeyId.Text, TextBoxAWSSecretAccessKey.Text)
        UserRequestHeaders.Add("Authorization", AuthorizationValue)

        RetBool = MakeAWSRequest(RequestURL, RequestMethod, UserRequestHeaders, PostData, ErrorNumber, ErrorDescription, LogData, FullRequestHeaders, ResponseStatusCode, ResponseStatusDescription, FullResponseHeaders, ResponseString)
        Debug.Print(LogData)

        If RetBool = True Then
            DialogOutput.Text = "Success"
            DialogOutput.TextBoxOutput.Text = ResponseString
            DialogOutput.ShowDialog(Me)
        Else
            ResponseMessage = FormatLogData(RequestURL, RequestMethod, FullRequestHeaders, ResponseStatusCode, ResponseStatusDescription, FullResponseHeaders, ResponseString, ErrorNumber, ErrorDescription)
            DialogOutput.Text = "Error"
            DialogOutput.TextBoxOutput.Text = ResponseMessage
            DialogOutput.ShowDialog(Me)
        End If

        sender.Enabled = True
    End Sub

End Class
