﻿Imports System.Windows.Forms

Public Class FormDownload

    Dim WithEvents MyDownload As SprightlySoftAWS.S3.Download
    Dim WithEvents DownloadBackgroundWorker As System.ComponentModel.BackgroundWorker

    Private Sub FormDownload_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        MyDownload = New SprightlySoftAWS.S3.Download

        DownloadBackgroundWorker = New System.ComponentModel.BackgroundWorker

        ProgressBarTransfered.Value = 0
        LabelBytesTransfered.Text = "0 bytes / 0 bytes"

    End Sub

    Private Sub EnableDisableStart()
        TextBoxDownloadBucketName.Enabled = False
        TextBoxDownloadKeyName.Enabled = False
        TextBoxDownloadFileName.Enabled = False
        ButtonDownloadFile.Enabled = False

        ButtonAbort.Enabled = True
        ProgressBarTransfered.Enabled = True
        LabelBytesTransfered.Enabled = True

        ProgressBarTransfered.Value = 0
        LabelBytesTransfered.Text = "0 bytes / 0 bytes"
    End Sub

    Private Sub EnableDisableEnd()
        TextBoxDownloadBucketName.Enabled = True
        TextBoxDownloadKeyName.Enabled = True
        TextBoxDownloadFileName.Enabled = True
        ButtonDownloadFile.Enabled = True

        ButtonAbort.Enabled = False
        ProgressBarTransfered.Enabled = False
        LabelBytesTransfered.Enabled = False

        ProgressBarTransfered.Value = 0
        LabelBytesTransfered.Text = "0 bytes / 0 bytes"
    End Sub

    Private Sub ButtonDownloadFile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonDownloadFile.Click
        EnableDisableStart()

        If InStr(TextBoxDownloadFileName.Text, "\") > 0 AndAlso System.IO.Directory.Exists(System.IO.Path.GetDirectoryName(TextBoxDownloadFileName.Text)) = True Then

            Dim RequestURL As String
            RequestURL = MyDownload.BuildS3RequestURL(True, "s3.amazonaws.com", TextBoxDownloadBucketName.Text, TextBoxDownloadKeyName.Text, "")

            Dim RequestMethod As String = "GET"

            Dim ExtraRequestHeaders As New Dictionary(Of String, String)
            ExtraRequestHeaders.Add("x-amz-date", DateTime.UtcNow.ToString("r"))

            Dim AuthorizationValue As String
            AuthorizationValue = MyDownload.GetS3AuthorizationValue(RequestURL, RequestMethod, ExtraRequestHeaders, FormMain.TextBoxAWSAccessKeyId.Text, FormMain.TextBoxAWSSecretAccessKey.Text)
            ExtraRequestHeaders.Add("Authorization", AuthorizationValue)

            'Use a hash table to pass parameters to the function in the BackgroundWorker.
            Dim DownloadHashTable As New Hashtable
            DownloadHashTable.Add("RequestURL", RequestURL)
            DownloadHashTable.Add("RequestMethod", RequestMethod)
            DownloadHashTable.Add("ExtraRequestHeaders", ExtraRequestHeaders)
            DownloadHashTable.Add("LocalFileName", TextBoxDownloadFileName.Text)

            'Run the DownloadFile function in a BackgroundWorker process.  Downloading a large
            'file may take a long time.  Running the process in a BackgroundWorker will prevent
            'the Window from locking up.
            DownloadBackgroundWorker.RunWorkerAsync(DownloadHashTable)

        Else
            EnableDisableEnd()
            MessageBox.Show("The local file path does not exist.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End If

    End Sub

    Private Sub DownloadBackgroundWorker_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles DownloadBackgroundWorker.DoWork
        'Call the DownloadFile function and set the result.  When the function is complete
        'the RunWorkerCompleted event will fire.  Use the parameters that were passed in the 
        'hash table.
        e.Result = MyDownload.DownloadFile(e.Argument("RequestURL"), e.Argument("RequestMethod"), e.Argument("ExtraRequestHeaders"), e.Argument("LocalFileName"), False)
    End Sub

    Private Sub DownloadBackgroundWorker_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles DownloadBackgroundWorker.RunWorkerCompleted

        Debug.Print("")
        Debug.Print(MyDownload.LogData)
        Debug.Print("")

        EnableDisableEnd()

        If e.Result = True Then
            MessageBox.Show("Download complete.", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information)
        Else
            'Delete a partially downloaded file
            If System.IO.File.Exists(TextBoxDownloadFileName.Text) = True Then
                System.IO.File.Delete(TextBoxDownloadFileName.Text)
            End If

            'Show the error message.
            Dim ResponseMessage As String

            If MyDownload.ResponseString = "" Then
                ResponseMessage = MyDownload.ErrorDescription
            Else
                Dim XmlDoc As New System.Xml.XmlDocument
                XmlDoc.LoadXml(MyDownload.ResponseString)

                Dim XmlNode As System.Xml.XmlNode
                XmlNode = XmlDoc.SelectSingleNode("/Error/Message")

                ResponseMessage = XmlNode.InnerText
            End If

            MessageBox.Show(ResponseMessage, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)

        End If

    End Sub

    Private Sub ButtonAbort_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonAbort.Click
        MyDownload.Abort()
    End Sub

    Private Sub MyDownload_ProgressChangedEvent() Handles MyDownload.ProgressChangedEvent
        If Me.InvokeRequired = True Then
            Me.Invoke(New MethodInvoker(AddressOf MyDownload_ProgressChangedEvent))
        Else
            'Set the progress bar when the ProgressChangedEvent is fired.
            If MyDownload.BytesTotal > 0 Then
                ProgressBarTransfered.Value = (MyDownload.BytesTransfered / MyDownload.BytesTotal) * 100

                Dim MyHelper As New SprightlySoftAWS.S3.Helper
                LabelBytesTransfered.Text = MyHelper.FormatByteSize(MyDownload.BytesTransfered) & " / " & MyHelper.FormatByteSize(MyDownload.BytesTotal)
            End If
        End If
    End Sub

End Class
