﻿Imports System.Windows.Forms

Public Class FormDownloadEstimate

    Dim WithEvents MyCalculateHash As SprightlySoftAWS.S3.CalculateHash
    Dim WithEvents MyDownload As SprightlySoftAWS.S3.DownloadEstimate
    Dim WithEvents DownloadBackgroundWorker As System.ComponentModel.BackgroundWorker
    Dim WithEvents ETagBackgroundWorker As System.ComponentModel.BackgroundWorker
    Dim TimerUpdateStatus As System.Timers.Timer
    Dim ExistingFileSize As Double

    Private Sub FormDownload_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        MyCalculateHash = New SprightlySoftAWS.S3.CalculateHash
        MyDownload = New SprightlySoftAWS.S3.DownloadEstimate

        DownloadBackgroundWorker = New System.ComponentModel.BackgroundWorker
        ETagBackgroundWorker = New System.ComponentModel.BackgroundWorker

        'use a timer to show the current speed and time remaining
        TimerUpdateStatus = New System.Timers.Timer
        AddHandler TimerUpdateStatus.Elapsed, AddressOf TimerUpdateStatus_Tick
        TimerUpdateStatus.Interval = 500
    End Sub

    Private Sub EnableDisableStart()
        TextBoxDownloadBucketName.Enabled = False
        TextBoxDownloadKeyName.Enabled = False
        TextBoxDownloadFileName.Enabled = False
        ButtonDownloadFile.Enabled = False
        ButtonAbort.Enabled = True

        ToolStripProgressBar.Value = 0
        ToolStripStatusLabelBytes.Text = ""
        ToolStripStatusLabelSpeed.Text = ""
        ToolStripStatusLabelElapsed.Text = ""
        ToolStripStatusLabelRemaining.Text = ""

        ToolStripProgressBar.Visible = True
        ToolStripStatusLabelBytes.Visible = True
        ToolStripStatusLabelSpeed.Visible = True
        ToolStripStatusLabelElapsed.Visible = True
        ToolStripStatusLabelRemaining.Visible = True

        TimerUpdateStatus.Enabled = True
    End Sub

    Private Sub EnableDisableEnd()
        TextBoxDownloadBucketName.Enabled = True
        TextBoxDownloadKeyName.Enabled = True
        TextBoxDownloadFileName.Enabled = True
        ButtonDownloadFile.Enabled = True
        ButtonAbort.Enabled = False

        ToolStripProgressBar.Visible = False
        ToolStripStatusLabelBytes.Visible = False
        ToolStripStatusLabelSpeed.Visible = False
        ToolStripStatusLabelElapsed.Visible = False
        ToolStripStatusLabelRemaining.Visible = False

        TimerUpdateStatus.Enabled = False
    End Sub

    Private Sub ButtonDownloadFile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonDownloadFile.Click
        EnableDisableStart()

        If InStr(TextBoxDownloadFileName.Text, "\") > 0 AndAlso System.IO.Directory.Exists(System.IO.Path.GetDirectoryName(TextBoxDownloadFileName.Text)) = True Then

            ExistingFileSize = 0
            Dim ExtraRequestHeaders As New Dictionary(Of String, String)
            Dim RequestURL As String
            Dim RequestMethod As String
            Dim AuthorizationValue As String
            Dim AppendToFile As Boolean = False
            Dim SkipDownload As Boolean = False

            If System.IO.File.Exists(TextBoxDownloadFileName.Text) = True Then
                'The file to download already exists.  Populate the conflict form and ask
                'the user what to do.

                Dim LocalFileInfo As New System.IO.FileInfo(TextBoxDownloadFileName.Text)
                ExistingFileSize = LocalFileInfo.Length
                FormConflictDownload.TextBoxLocalFileName.Text = TextBoxDownloadFileName.Text
                FormConflictDownload.TextBoxLocalSize.Text = FormatNumber(LocalFileInfo.Length, 0) & " bytes"
                FormConflictDownload.TextBoxLocalDate.Text = LocalFileInfo.LastWriteTime

                'kick off a process to calculate the etag.  A large file may take a long time to calculate.
                FormConflictDownload.TextBoxLocalETag.Text = "calculating...."
                Dim ETagHashTable As New Hashtable
                ETagHashTable.Add("LocalFileName", TextBoxDownloadFileName.Text)
                ETagBackgroundWorker.RunWorkerAsync(ETagHashTable)

                'get the properties of the Amazon file
                Dim MyREST As New SprightlySoftAWS.REST

                RequestURL = MyREST.BuildS3RequestURL(True, "s3.amazonaws.com", TextBoxDownloadBucketName.Text, TextBoxDownloadKeyName.Text, "")
                RequestMethod = "HEAD"

                ExtraRequestHeaders.Add("x-amz-date", DateTime.UtcNow.ToString("r"))

                AuthorizationValue = MyREST.GetS3AuthorizationValue(RequestURL, RequestMethod, ExtraRequestHeaders, FormMain.TextBoxAWSAccessKeyId.Text, FormMain.TextBoxAWSSecretAccessKey.Text)
                ExtraRequestHeaders.Add("Authorization", AuthorizationValue)

                Dim RetBool As Boolean
                RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, ExtraRequestHeaders, "")

                If RetBool = True Then
                    FormConflictDownload.TextBoxRemoteBucketName.Text = TextBoxDownloadBucketName.Text
                    FormConflictDownload.TextBoxRemoteKeyName.Text = TextBoxDownloadKeyName.Text
                    FormConflictDownload.TextBoxRemoteSize.Text = FormatNumber(MyREST.ResponseHeaders("Content-Length"), 0) & " bytes"
                    FormConflictDownload.TextBoxRemoteAmazonDate.Text = CDate(MyREST.ResponseHeaders("Last-Modified"))
                    FormConflictDownload.TextBoxRemoteETag.Text = Replace(MyREST.ResponseHeaders("ETag"), """", "")
                Else
                    EnableDisableEnd()
                    MessageBox.Show(MyREST.ErrorDescription, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
                    Exit Sub
                End If

                'only allow resume if the Amazon file is larger
                If CDbl(MyREST.ResponseHeaders("Content-Length")) > LocalFileInfo.Length Then
                    FormConflictDownload.ButtonResume.Enabled = True
                Else
                    FormConflictDownload.ButtonResume.Enabled = False
                End If

                'show the conflict form
                Dim DialogResult As System.Windows.Forms.DialogResult
                DialogResult = FormConflictDownload.ShowDialog(Me)

                If DialogResult = Windows.Forms.DialogResult.Retry Then
                    'only download the remainder of the file.   Add a range header to the request.
                    ExtraRequestHeaders.Add("Range", "bytes=" & LocalFileInfo.Length & "-" & CDbl(MyREST.ResponseHeaders("Content-Length")) - 1)
                    AppendToFile = True
                ElseIf DialogResult = Windows.Forms.DialogResult.Yes Then
                    'do nothing
                Else
                    SkipDownload = True
                End If

            End If


            If SkipDownload = True Then
                EnableDisableEnd()
                MessageBox.Show("Download skipped.", "Skipped", MessageBoxButtons.OK, MessageBoxIcon.Information)
            Else

                RequestURL = MyDownload.BuildS3RequestURL(True, "s3.amazonaws.com", TextBoxDownloadBucketName.Text, TextBoxDownloadKeyName.Text, "")
                RequestMethod = "GET"

                ExtraRequestHeaders = New Dictionary(Of String, String)
                ExtraRequestHeaders.Add("x-amz-date", DateTime.UtcNow.ToString("r"))

                AuthorizationValue = MyDownload.GetS3AuthorizationValue(RequestURL, RequestMethod, ExtraRequestHeaders, FormMain.TextBoxAWSAccessKeyId.Text, FormMain.TextBoxAWSSecretAccessKey.Text)
                ExtraRequestHeaders.Add("Authorization", AuthorizationValue)

                'Use a hash table to pass parameters to the function in the BackgroundWorker.
                Dim DownloadHashTable As New Hashtable
                DownloadHashTable.Add("RequestURL", RequestURL)
                DownloadHashTable.Add("RequestMethod", RequestMethod)
                DownloadHashTable.Add("ExtraRequestHeaders", ExtraRequestHeaders)
                DownloadHashTable.Add("LocalFileName", TextBoxDownloadFileName.Text)
                DownloadHashTable.Add("AppendToFile", AppendToFile)

                'Run the DownloadFile function in a BackgroundWorker process.  Downloading a large
                'file may take a long time.  Running the process in a BackgroundWorker will prevent
                'the Window from locking up.
                DownloadBackgroundWorker.RunWorkerAsync(DownloadHashTable)
            End If

        Else
            EnableDisableEnd()
            MessageBox.Show("The local file path does not exist.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End If

    End Sub

    Private Sub DownloadBackgroundWorker_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles DownloadBackgroundWorker.DoWork
        'Call the DownloadFile function and set the result.  When the function is complete
        'the RunWorkerCompleted event will fire.  Use the parameters that were passed in the 
        'hash table.
        e.Result = MyDownload.DownloadFile(e.Argument("RequestURL"), e.Argument("RequestMethod"), e.Argument("ExtraRequestHeaders"), e.Argument("LocalFileName"), False)
    End Sub

    Private Sub DownloadBackgroundWorker_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles DownloadBackgroundWorker.RunWorkerCompleted

        Debug.Print("")
        Debug.Print(MyDownload.LogData)
        Debug.Print("")

        EnableDisableEnd()

        If e.Result = True Then
            MessageBox.Show("Download complete.", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information)
        Else
            'Show the error message.
            Dim ResponseMessage As String

            If MyDownload.ResponseString = "" Then
                ResponseMessage = MyDownload.ErrorDescription
            Else
                Dim XmlDoc As New System.Xml.XmlDocument
                XmlDoc.LoadXml(MyDownload.ResponseString)

                Dim XmlNode As System.Xml.XmlNode
                XmlNode = XmlDoc.SelectSingleNode("/Error/Message")

                ResponseMessage = XmlNode.InnerText
            End If

            MessageBox.Show(ResponseMessage, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End If

    End Sub

    Private Sub ButtonAbort_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonAbort.Click
        MyDownload.Abort()
    End Sub

    Private Sub MyDownload_ProgressChangedEvent() Handles MyDownload.ProgressChangedEvent
        If Me.InvokeRequired = True Then
            Me.Invoke(New MethodInvoker(AddressOf MyDownload_ProgressChangedEvent))
        Else
            'update the progress bar when the ProgressChangedEvent is fired.
            If MyDownload.BytesTotal > 0 Then
                ToolStripProgressBar.Value = ((MyDownload.BytesTransfered + ExistingFileSize) / (MyDownload.BytesTotal + ExistingFileSize)) * 100

                Dim MyHelper As New SprightlySoftAWS.S3.Helper
                ToolStripStatusLabelBytes.Text = MyHelper.FormatByteSize(MyDownload.BytesTransfered + ExistingFileSize) & " / " & MyHelper.FormatByteSize(MyDownload.BytesTotal + ExistingFileSize)
            End If
        End If
    End Sub

    Private Sub TimerUpdateStatus_Tick()

        If Me.InvokeRequired = True Then
            Me.Invoke(New MethodInvoker(AddressOf TimerUpdateStatus_Tick))
        Else
            'get the current speed, time elapsed and time remaining from the DownloadEstimate class
            ToolStripStatusLabelSpeed.Text = FormatNumber(MyDownload.CurrentKBpsSpeed, 2) & " KB/s"

            Dim ElapsedTimeSpan As TimeSpan
            ElapsedTimeSpan = New TimeSpan(0, 0, MyDownload.SecondsElapsed)
            ToolStripStatusLabelElapsed.Text = ElapsedTimeSpan.ToString & " elapsed"

            Dim RemainingTimeSpan As TimeSpan
            RemainingTimeSpan = New TimeSpan(0, 0, MyDownload.SecondsRemaining)
            ToolStripStatusLabelRemaining.Text = RemainingTimeSpan.ToString & " remaining"
        End If

    End Sub

    Private Sub ETagBackgroundWorker_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles ETagBackgroundWorker.DoWork
        e.Result = MyCalculateHash.CalculateETagFromFile(e.Argument("LocalFileName"))
    End Sub

    Private Sub ETagBackgroundWorker_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles ETagBackgroundWorker.RunWorkerCompleted
        'show the etag in the conflict form
        FormConflictDownload.TextBoxLocalETag.Text = e.Result
    End Sub

End Class
